﻿using Common;
using Dapper;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using RedisHelp;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using TGood.Xian.LogManager;
using WxPayAPI.Models;
using wxWebAPI.business;
using wxWebAPI.Common;
using wxWebAPI.Models;
using wxWebAPI.Models.cardapplet;
using wxWebAPI.Models.UMS;

namespace WxPayAPI
{
    public class PayLogic
    {
        static RedisHelper redis12 = new RedisHelper((int)EnumUserRedisNum.Reserve);
        public static OperatorData manuOperator = null;
        public static OperatorData teldOperator = redis12.HashGet<OperatorData>(RedisKey.ReverseOperator, ConfigurationManager.AppSettings["ZDYLOperatorID"]);

        public static string ControlAPI = ConfigurationManager.AppSettings["controlAPI"];
        public const string DemoQRCode = "888888888888888";
        public static readonly string RedisConnectionString = ConfigurationManager.ConnectionStrings["RedisExchangeHosts"].ConnectionString;
        internal static string StrategyAPI = ConfigurationManager.AppSettings["StrategyAPI"];
        #region 微信小程序登录
        public string Login(LoginInfo loginInfo)
        {
            string userToken = string.Empty;
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                string appid = WxPayConfig.MINA_APPID;
                string secret = WxPayConfig.MINA_APPSECRET;


                if (!string.IsNullOrEmpty(loginInfo.CompanyID))
                {
                    var wxSubMCHInfo = redis.HashGet<WXSubMCHInfo>(RedisKey.SubMCH, loginInfo.CompanyID);
                    if (wxSubMCHInfo != null
                        && !string.IsNullOrEmpty(wxSubMCHInfo.MINA_APPID) && !string.IsNullOrEmpty(wxSubMCHInfo.MINA_APPSECRET))
                    {
                        appid = wxSubMCHInfo.MINA_APPID;
                        secret = wxSubMCHInfo.MINA_APPSECRET;
                    }
                }

                else if (!string.IsNullOrEmpty(loginInfo.appName) && loginInfo.appName.ToUpper() == "RUIZHIYI")
                {
                    appid = WxPayConfig.RZY_APPID;
                    secret = WxPayConfig.RZY_APPSECRET;
                }

                else if (!string.IsNullOrEmpty(loginInfo.appName) && loginInfo.appName.ToUpper() == "ZDYL")
                {
                    appid = WxPayConfig.ZDYLAPPID_MM;
                    secret = WxPayConfig.ZDYLAPPSECRET_MM;
                }

                //构造获取openid的url
                WxPayData data = new WxPayData();
                data.SetValue("appid", appid);
                data.SetValue("secret", secret);
                data.SetValue("js_code", loginInfo.Code);
                data.SetValue("grant_type", "authorization_code");
                string url = "https://api.weixin.qq.com/sns/jscode2session?" + data.ToUrl();

                //请求url以获取数据
                string apiResult = HttpService.Get(url);
                //LogEvent.WebAPILoger.Debug(this.GetType().ToString(), "GetOpenidAndAccessTokenFromCode response : " + apiResult);

                var jsonData = JObject.Parse(apiResult);
                if (jsonData.GetValue("openid") != null)
                {
                    //以openid为Key，将openid、assess_key信息存入redis
                    userToken = Guid.NewGuid().ToString("N");
                    var openid = jsonData.GetValue("openid").ToString();
                    SaveTokenToRedis(userToken, openid);

                    var sessionKey = jsonData.GetValue("session_key").ToString();
                    SaveSessionKeyToRedis(openid, sessionKey);
                }
                else
                {

                    LogEvent.WebAPILoger.Error("PayLogic - Login - apiResult:" + apiResult);
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("PayLogic - Login", ex);
            }

            return userToken;
        }

        private void SaveTokenToRedis(string userToken, string openid)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            var oldToken = redis.HashGetString(RedisKey.TokenReverse, openid);
            if (!string.IsNullOrEmpty(oldToken))
            {
                redis.HashDelete(RedisKey.Token, oldToken);
            }
            redis.HashSet(RedisKey.TokenReverse, openid, userToken);
            redis.HashSet(RedisKey.Token, userToken, openid);
            LogEvent.WebAPILoger.Info($"usertoken存入redis time = {DateTime.Now} , usertoken = {userToken},openid={openid}");
        }

        private void SaveSessionKeyToRedis(string openid, string sessionKey)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            redis.HashSet(RedisKey.SessionKey, openid, sessionKey);
        }

        #endregion

        #region 退款业务
        public string RefundFuwushang(WXRefund refundParameter)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            if (refundParameter.CompanyID == "900000000")
            {
                redis.ListLeftPush(RedisKey.WXRefund, refundParameter);
                return "";
            }
            string apiResult = RefundQuery(refundParameter).ToJson();
            // 将汉字的Unicode转字符串
            var apiResultConvert = new Regex(@"\\u([0-9A-F]{4})", RegexOptions.IgnoreCase | RegexOptions.Compiled)
              .Replace(apiResult, x => string.Empty + Convert.ToChar(Convert.ToUInt16(x.Result("$1"), 16)));
            LogEvent.WebAPILoger.Info($"HandleWXRefundResult apiresult = {apiResultConvert},refundparameter = {refundParameter}");

            if (apiResult != null)
            {
                JObject jsonResult = JObject.Parse(JsonConvert.DeserializeObject(apiResult).ToString());
                if (jsonResult.Property("err_code") != null && jsonResult.GetValue("err_code").ToString().ToUpper() == "REFUNDNOTEXIST")
                {
                    LogEvent.WebAPILoger.Info($"refundparameter = {refundParameter}");
                    redis.ListLeftPush(RedisKey.WXRefund, refundParameter);
                    return "";
                }
                else if (jsonResult.GetValue("return_code").ToString().ToUpper() == "SUCCESS"
                    && jsonResult.GetValue("result_code").ToString().ToUpper() == "SUCCESS")
                {
                    if (jsonResult.GetValue("refund_status_0").ToString().ToUpper() == "SUCCESS" || (int)jsonResult.GetValue("refund_count") >= 1)
                    {
                        LogEvent.WebAPILoger.Info($"此订单已经发生过退款。refundParameter.ToString={refundParameter.ToString()}");
                        return "";
                    }
                    else
                    {
                        redis.ListLeftPush(RedisKey.WXRefund, refundParameter);
                    }
                }
                else
                {
                    LogEvent.WebAPILoger.Error($"此订单查询退款记录失败。refundParameter={refundParameter.ToString()}。apiResult={apiResult}。");
                }
            }
            return "";
        }
        public string Refund(WXRefund refundParameter)
        {
            string resultString = "";

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            switch (refundParameter.PaymentType)
            {
                case EnumPaymentType.小程序微信:

                    WXPayData refundResult = RefundRunFuwushang(refundParameter);
                    resultString = refundResult.ToJson();
                    break;

                case EnumPaymentType.银商支付宝:
                    //resultString = ALIRefund(refundParameter);

                    break;

                case EnumPaymentType.威富通微信:
                    //resultString = WFTRefund(refundParameter);

                    break;

                default:
                    break;
            }

            return resultString;
        }
        #endregion

        #region 微信服务商退款

        /// <summary>
        /// 查询退款信息
        /// </summary>
        /// <param name="refundParameter"></param>
        /// <returns></returns>
        public WXPayData RefundQuery(WXRefund refundParameter)
        {
            string key = WxPayConfig.KEY;
            string url = "https://api.mch.weixin.qq.com/pay/refundquery";
            int timeOut = 6;
            WXSubMCHInfo wxSubMCHInfo = GetSubAPPIDandMCHID(refundParameter.CompanyID);

            WXPayData data = new WXPayData();
            //微信订单号存在的条件下，则已微信订单号为准
            if (!string.IsNullOrEmpty(refundParameter.Transaction_id))
            {
                data.SetValue("transaction_id", refundParameter.Transaction_id);
            }
            else//微信订单号不存在，才根据商户订单号去退款
            {
                data.SetValue("out_trade_no", refundParameter.Out_trade_no);
            }

            if (!string.IsNullOrEmpty(refundParameter.appname) && refundParameter.appname.ToUpper() == "ZDYL")
            {
                data.SetValue("appid", WxPayConfig.ZDYLAPPID);
                data.SetValue("mch_id", WxPayConfig.ZDYLMCHID);
                key = WxPayConfig.ZDYLKEY;
            }
            else
            {
                data.SetValue("appid", WxPayConfig.APPID);
                data.SetValue("mch_id", WxPayConfig.MCHID);
                key = WxPayConfig.KEY;
            }
            data.SetValue("sub_appid", wxSubMCHInfo.MINA_APPID);
            data.SetValue("sub_mch_id", wxSubMCHInfo.MCHID);
            data.SetValue("nonce_str", GenerateNonceStr());
            data.SetValue("sign", data.MakeSign(key));

            string xml = data.ToXml();

            LogEvent.NotificationLoger.Info($"Refundquery - Refundquery request : {xml}");
            string response = HttpService.Post(xml, url, false, timeOut);
            LogEvent.NotificationLoger.Info($"Refundquery - Refundquery response : {response}");

            //将xml格式的结果转换为对象以返回
            WXPayData result = new WXPayData();
            result.FromXml(response, key);

            //Log.Info("Refund", "Refund process complete, result : " + result.ToXml());
            LogEvent.NotificationLoger.Info($"Refundquery - Refundquery process complete, result : {result.ToXml()}");

            return result;
        }
        private WXPayData RefundRunFuwushang(WXRefund refundParameter)
        {
            //Log.Info("Refund", "Refund is processing...");
            LogEvent.NotificationLoger.Info($"Refund is processing...");
            string url = "https://api.mch.weixin.qq.com/secapi/pay/refund";
            int timeOut = 6;

            // 确定此设备的费用归属哪家子商户
            WXSubMCHInfo wxSubMCHInfo = GetSubAPPIDandMCHID(refundParameter.CompanyID);

            WXPayData data = new WXPayData();
            //微信订单号存在的条件下，则已微信订单号为准
            if (!string.IsNullOrEmpty(refundParameter.Transaction_id))
            {
                data.SetValue("transaction_id", refundParameter.Transaction_id);
            }
            else//微信订单号不存在，才根据商户订单号去退款
            {
                data.SetValue("out_trade_no", refundParameter.Out_trade_no);
            }

            //订单总金额
            data.SetValue("total_fee", refundParameter.Total_fee);
            //退款金额
            data.SetValue("refund_fee", refundParameter.Refund_fee);
            if (refundParameter.RefundType == EnumRefundType.人工操作)
            {
                data.SetValue("out_refund_no", Guid.NewGuid().ToString("N"));
            }
            else
            {
                //随机生成商户退款单号
                data.SetValue("out_refund_no", refundParameter.Out_trade_no + "1");
            }
            //操作员，默认为商户号

            //若商户传入，会在下发给用户的退款消息中体现退款原因
            //TODO:此处可以考虑传订单结束原因
            data.SetValue("refund_desc", "费用剩余");
            data.SetValue("sub_appid", wxSubMCHInfo.MINA_APPID);
            data.SetValue("sub_mch_id", wxSubMCHInfo.MCHID);
            data.SetValue("nonce_str", GenerateNonceStr());
            if (!string.IsNullOrEmpty(refundParameter.appname) && refundParameter.appname.ToUpper() == "ZDYL")
            {
                data.SetValue("appid", WxPayConfig.ZDYLAPPID_MM);
                data.SetValue("mch_id", WxPayConfig.ZDYLMCHID);
                data.SetValue("op_user_id", WxPayConfig.ZDYLMCHID);
                data.SetValue("sign", data.MakeSign(WxPayConfig.ZDYLKEY));
            }
            else
            {
                data.SetValue("appid", WxPayConfig.APPID);
                data.SetValue("mch_id", WxPayConfig.MCHID);
                data.SetValue("op_user_id", WxPayConfig.MCHID);
                data.SetValue("sign", data.MakeSign(WxPayConfig.KEY));
            }
            string xml = data.ToXml();

            //Log.Debug("WxPayApi", "Refund request : " + xml);
            LogEvent.NotificationLoger.Info($"RefundRunFuwushang - Refund request : {xml}");
            string response = HttpService.Post(xml, url, true, timeOut, refundParameter.appname);
            //Log.Debug("WxPayApi", "Refund response : " + response);
            LogEvent.NotificationLoger.Info($"RefundRunFuwushang - Refund response : {response}");

            //将xml格式的结果转换为对象以返回
            WXPayData result = new WXPayData();
            if (!string.IsNullOrEmpty(refundParameter.appname) && refundParameter.appname.ToUpper() == "ZDYL")
            {
                result.FromXml(response, WxPayConfig.ZDYLKEY);
            }
            else
            {
                result.FromXml(response, WxPayConfig.KEY);
            }
            //Log.Info("Refund", "Refund process complete, result : " + result.ToXml());
            LogEvent.NotificationLoger.Info($"RefundRunFuwushang - Refund process complete, result : {result.ToXml()}");

            return result;
        }

        public string GetCertficates(string mch_id)
        {
            string url = "https://api.mch.weixin.qq.com/risk/getcertficates";
            int timeOut = 6;

            WXPayData data = new WXPayData();
            data.SetValue("mch_id", mch_id);
            data.SetValue("nonce_str", GenerateNonceStr());
            data.SetValue("sign_type", "HMAC-SHA256");
            data.SetValue("sign", data.MakeSign_HMACSHA256(WxPayConfig.ZDYLKEY));

            string xml = data.ToXml();
            string response = HttpService.Post(xml, url, false, timeOut);

            LogEvent.NotificationLoger.Info($"RefundRunFuwushang - Refund process complete, result : {response}");

            return response;
        }

        #endregion

        #region 微信服务商统一下单

        public string NewWXMINAPayFuwushang(WXMINAPayParameter param)
        {
            param.deviceCode = WechatUtility.ShenSiFilter(param.deviceCode);
            string openid = WechatUtility.GetOpenIDByUserToken(param.userToken);
            if (string.IsNullOrEmpty(openid))
            {
                throw new ArgumentException($"param.userToken={param.userToken},此token获取不到openid");
            }
            if (!string.IsNullOrEmpty(param.phone))
            {
                new UserLogic().SaveUserAndPhone(openid, param.phone, "nickName");
            }

            // 保存openid和昵称的对应关系，昵称在微信支付中attch传输的话，经常因为非utf-8字符导致下单或回调失败
            SaveIDandNickName(openid, param.nickName);
            param.nickName = "nickName";
            if (!param.deviceCode.Contains("|"))
            {
                param.deviceCode = string.Format("{0}|{1}",param.deviceCode.Substring(0, param.deviceCode.Length - 2),param.deviceCode.Substring(param.deviceCode.Length - 2, 2));
            }
            // string companyID = "";
            // 根据设备确定此设备的费用归属哪家子商户
            LCO_Station station = WechatUtility.GetStation(param.deviceCode);
            //TODO:小程序需要传参增加一个本身的appid参数
            //param.companyID = station.CompanyID;//运营商ID以device对应的station中的companyID为准，前端传给的不用
            WXSubMCHInfo wxSubMCHInfo = GetSubAPPIDandMCHID(station.CompanyID);
            if (wxSubMCHInfo == null || string.IsNullOrEmpty(wxSubMCHInfo.MCHID))
            {
                return "子商户信息不存在";
            }
            if (wxSubMCHInfo.MINA_APPID != param.APPID)
            {
                if (param.APPID == WxPayAPI.WxPayConfig.MINA_APPID)
                {
                    wxSubMCHInfo.MINA_APPSECRET = WxPayAPI.WxPayConfig.MINA_APPSECRET;
                    wxSubMCHInfo.MINA_APPID = WxPayAPI.WxPayConfig.MINA_APPID;
                }
                else if (param.APPID == WxPayAPI.WxPayConfig.ZDYLAPPID_MM)
                {
                    wxSubMCHInfo.MINA_APPSECRET = WxPayAPI.WxPayConfig.ZDYLAPPSECRET_MM;
                    wxSubMCHInfo.MINA_APPID = WxPayAPI.WxPayConfig.ZDYLAPPID_MM;
                }
                else
                {
                    if (param.AppName != "APP")
                    {
                        LogEvent.WebAPILoger.Error($"此设备不属于此程序对应设备！param={param.ToString()}");
                        return "";
                    }
                }
            }


            // 统一下单
            WXPayData payData = GetUnifiedOrderResultFuwushang(openid, param, wxSubMCHInfo, station.CompanyID);

            // 保存预付款ID，为了主动推送模板消息
            SavePrepayID(openid, payData.GetValue("prepay_id").ToString());

            var parameters = GetJsApiParametersFuwushang(payData, wxSubMCHInfo, param.AppName);

            return parameters;
        }

        public string WXMINAPayFuwushang(WXMINAPayParameter param)
        {
            string openid = WechatUtility.GetOpenIDByUserToken(param.userToken);
            if (string.IsNullOrEmpty(openid))
            {
                throw new ArgumentException($"param.userToken={param.userToken},此token获取不到openid");
            }


            // 保存openid和昵称的对应关系，昵称在微信支付中attch传输的话，经常因为非utf-8字符导致下单或回调失败
            SaveIDandNickName(openid, param.nickName);
            param.nickName = "nickName";

            // 根据设备确定此设备的费用归属哪家子商户
            LCO_Station station = WechatUtility.GetStation(param.deviceCode);
            //TODO:小程序需要传参增加一个本身的appid参数
            //param.companyID = station.CompanyID;//运营商ID以device对应的station中的companyID为准，前端传给的不用
            WXSubMCHInfo wxSubMCHInfo = GetSubAPPIDandMCHID(station.CompanyID);


            // 统一下单
            WXPayData payData = GetUnifiedOrderResultFuwushang(openid, param, wxSubMCHInfo, station.CompanyID);

            // 保存预付款ID，为了主动推送模板消息
            SavePrepayID(openid, payData.GetValue("prepay_id").ToString());

            var parameters = GetJsApiParametersFuwushang(payData, wxSubMCHInfo, param.AppName);

            return parameters;
        }

        /// <summary>
        /// 保存预付款ID，为了主动推送模板消息
        /// </summary>
        /// <param name="openID"></param>
        /// <param name="prepay_id"></param>
        private void SavePrepayID(string openID, string prepay_id)
        {
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                redis.HashSet(RedisKey.WXPrepayID, openID, prepay_id);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"SavePrepayID ,openID={openID},prepay_id={prepay_id}。", ex);
            }

        }

        public void SaveIDandNickName(string openID, string nickName)
        {
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                var wxSubMCHInfo = redis.HashSet(RedisKey.IDNickName, openID, nickName);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"SaveIDandNickName ,openID={openID},nickName={nickName}。", ex);
            }
        }

        /// <summary>
        /// 调用统一下单，获得下单结果
        /// </summary>
        /// <param name="openID"></param>
        /// <param name="stationCode"></param>
        /// <param name="deviceCode"></param>
        /// <param name="fee">下单费用，单位：分</param>
        /// <returns>统一下单结果</returns>
        /// <exception cref="">失败时抛异常WxPayException</exception>
        private WXPayData GetUnifiedOrderResultFuwushang(string openID, WXMINAPayParameter parameter, WXSubMCHInfo wxSubMCHInfo, string companyID)
        {
            string url = "https://api.mch.weixin.qq.com/pay/unifiedorder";
            int timeOut = 6;
            string key = WxPayConfig.KEY;
            //统一下单
            WXPayData data = new WXPayData();
            //自定义参数，可以为终端设备号(门店号或收银设备ID)，PC网页或公众号内支付可以传"WEB"
            data.SetValue("device_info", parameter.deviceCode);
            data.SetValue("body", string.IsNullOrEmpty(ChargeLogic.WXBody) ? "充电费用" : ChargeLogic.WXBody);

            data.SetValue("out_trade_no", GenerateOutTradeNo(wxSubMCHInfo.MCHID));
            data.SetValue("total_fee", parameter.fee);
            data.SetValue("time_start", DateTime.Now.ToString("yyyyMMddHHmmss"));
            data.SetValue("time_expire", DateTime.Now.AddMinutes(10).ToString("yyyyMMddHHmmss"));
            data.SetValue("goods_tag", "test");

            //data.SetValue("openid", openID);
            //终端ip
            data.SetValue("spbill_create_ip", WxPayConfig.IP);
            data.SetValue("nonce_str", GenerateNonceStr());
            data.SetValue("notify_url", WxPayConfig.WX_MINA_NOTIFY_URL_Fuwushang);//异步通知url


            //子商户小程序ID
            //data.SetValue("sub_appid", string.IsNullOrEmpty(wxSubMCHInfo.MINA_APPID) ? WxPayConfig.MINA_APPID : wxSubMCHInfo.MINA_APPID);

            //子商户商户号
            data.SetValue("sub_mch_id", wxSubMCHInfo.MCHID);
            if (parameter.AppName != "APP")
            {
                data.SetValue("sub_openid", openID);
                data.SetValue("trade_type", "JSAPI");

            }
            //小马小程序支付
            if (!string.IsNullOrEmpty(parameter.AppName) && (parameter.AppName.ToUpper() == "ZDYL" || parameter.AppName.ToUpper() == "RUIZHIYI"))
            {
                //服务商公众账号ID
                data.SetValue("appid", WxPayConfig.ZDYLAPPID);
                //服务商商户号
                data.SetValue("mch_id", WxPayConfig.ZDYLMCHID);
                data.SetValue("sub_appid", string.IsNullOrEmpty(parameter.companyID) ? WxPayConfig.ZDYLAPPID_MM : wxSubMCHInfo.MINA_APPID);
                if (!string.IsNullOrEmpty(parameter.phone))
                {
                    data.SetValue("attach", $"{companyID}-{parameter.AppName}-{parameter.phone}");
                }
                else
                {
                    data.SetValue("attach", $"{companyID}-{parameter.AppName}");
                }
                key = WxPayConfig.ZDYLKEY;
                data.SetValue("notify_url", WxPayConfig.ZDYLWX_MINA_NOTIFY_URL_Fuwushang);//异步通知url
                //签名
                data.SetValue("sign", data.MakeSign(WxPayConfig.ZDYLKEY));
            }
            //领充支付
            else
            {
                //服务商公众账号ID
                data.SetValue("appid", WxPayConfig.APPID);
                //服务商商户号
                data.SetValue("mch_id", WxPayConfig.MCHID);
                data.SetValue("sub_appid", string.IsNullOrEmpty(parameter.companyID) ? WxPayConfig.MINA_APPID : wxSubMCHInfo.MINA_APPID);
                data.SetValue("notify_url", WxPayConfig.WX_MINA_NOTIFY_URL_Fuwushang);//异步通知url
                if (!string.IsNullOrEmpty(parameter.phone))
                {
                    data.SetValue("attach", $"{companyID}-{parameter.AppName}-{parameter.phone}");
                }
                else
                {
                    data.SetValue("attach", $"{companyID}");
                }
                //签名
                data.SetValue("sign", data.MakeSign(WxPayConfig.KEY));
            }


            string xml = data.ToXml();
            Log.Debug("WxPayApi", "GetUnifiedOrderResultFuwushang request : " + xml);
            string response = HttpService.Post(xml, url, false, timeOut, parameter.AppName);
            Log.Debug("WxPayApi", "GetUnifiedOrderResultFuwushang response : " + response);

            WXPayData result = new WXPayData();

            result.FromXml(response, key);

            if (!result.IsSet("appid") || !result.IsSet("prepay_id") || result.GetValue("prepay_id").ToString() == "")
            {
                var error = $"GetUnifiedOrderResultFuwushang - UnifiedOrder response error! result = {result.ToPrintStr()}。sub_appid={data.GetValue("sub_appid")},sub_openid={openID}，sub_mch_id={wxSubMCHInfo.MCHID}，deviceInfo={parameter.deviceCode},CompanyID={parameter.companyID}。";
                Log.Error("GetUnifiedOrderResultFuwushang", error);
                LogEvent.WebAPILoger.Error(error);
                throw new WxPayException("GetUnifiedOrderResultFuwushang response error! Maybe sub_appid and sub_openid not match");
            }

            return result;
        }

        private WXPayData GetUnifiedCardResultFuwushang(string openID, cardPayParam parameter, WXSubMCHInfo wxSubMCHInfo, string companyID)
        {
            string url = "https://api.mch.weixin.qq.com/pay/unifiedorder";
            int timeOut = 6;

            //统一下单
            WXPayData data = new WXPayData();
            //自定义参数，可以为终端设备号(门店号或收银设备ID)，PC网页或公众号内支付可以传"WEB"
            data.SetValue("device_info", parameter.cardid);
            data.SetValue("body", string.IsNullOrEmpty(ChargeLogic.WXBody) ? "充值卡费用" : ChargeLogic.WXBody);
            data.SetValue("attach", $"{companyID}");
            data.SetValue("out_trade_no", GenerateOutTradeNo(wxSubMCHInfo.MCHID));
            data.SetValue("total_fee", parameter.fee);
            data.SetValue("time_start", DateTime.Now.ToString("yyyyMMddHHmmss"));
            data.SetValue("time_expire", DateTime.Now.AddMinutes(10).ToString("yyyyMMddHHmmss"));
            data.SetValue("goods_tag", "test");
            data.SetValue("trade_type", "JSAPI");
            //data.SetValue("openid", openID);
            //终端ip
            data.SetValue("spbill_create_ip", WxPayConfig.IP);
            data.SetValue("nonce_str", GenerateNonceStr());
            data.SetValue("notify_url", WxPayConfig.WX_Card_NOTIFY_URL_Fuwushang);//异步通知url
            //服务商公众账号ID
            data.SetValue("appid", WxPayConfig.APPID);
            //服务商商户号
            data.SetValue("mch_id", WxPayConfig.MCHID);

            //子商户小程序ID
            //data.SetValue("sub_appid", string.IsNullOrEmpty(wxSubMCHInfo.MINA_APPID) ? WxPayConfig.MINA_APPID : wxSubMCHInfo.MINA_APPID);

            data.SetValue("sub_appid", string.IsNullOrEmpty(companyID) ? WxPayConfig.MINA_APPID : wxSubMCHInfo.MINA_APPID);

            //子商户商户号
            data.SetValue("sub_mch_id", wxSubMCHInfo.MCHID);
            data.SetValue("sub_openid", openID);


            //签名
            data.SetValue("sign", data.MakeSign(WxPayConfig.KEY));

            string xml = data.ToXml();
            Log.Debug("WxPayApi", "GetUnifiedCardResultFuwushang request : " + xml);
            string response = HttpService.Post(xml, url, false, timeOut);
            Log.Debug("WxPayApi", "GetUnifiedCardResultFuwushang response : " + response);

            WXPayData result = new WXPayData();
            result.FromXml(response, WxPayConfig.KEY);

            if (!result.IsSet("appid") || !result.IsSet("prepay_id") || result.GetValue("prepay_id").ToString() == "")
            {
                var error = $"GetUnifiedCardResultFuwushang - UnifiedOrder response error! result = {result.ToPrintStr()}。sub_appid={data.GetValue("sub_appid")},sub_openid={openID}，sub_mch_id={wxSubMCHInfo.MCHID}，deviceInfo={parameter.cardid},CompanyID={companyID}。";
                Log.Error("GetUnifiedCardResultFuwushang", error);
                LogEvent.WebAPILoger.Error(error);
                throw new WxPayException("GetUnifiedCardResultFuwushang response error! Maybe sub_appid and sub_openid not match");
            }

            return result;
        }

        private string GetJsApiParametersFuwushang(WXPayData unifiedOrderResult, WXSubMCHInfo wxSubMCHInfo, string appname)
        {
            WXPayData jsApiParam = new WXPayData();
            //此处要传子商户小程序的APPID，不能传服务商公众号的APPID
            jsApiParam.SetValue("appId", wxSubMCHInfo.MINA_APPID);
            jsApiParam.SetValue("timeStamp", GenerateTimeStamp());
            jsApiParam.SetValue("nonceStr", GenerateNonceStr());
            jsApiParam.SetValue("package", "prepay_id=" + unifiedOrderResult.GetValue("prepay_id"));
            jsApiParam.SetValue("signType", "MD5");
            if (appname == "ZDYL")
            {
                jsApiParam.SetValue("paySign", jsApiParam.MakeSign(WxPayConfig.ZDYLKEY));
            }
            else
            {
                jsApiParam.SetValue("paySign", jsApiParam.MakeSign(WxPayConfig.KEY));
            }
            string parameters = jsApiParam.ToJson();

            //LogEvent.WebAPILoger.Debug(this.GetType().ToString(), "Get jsApiParam : " + parameters);
            return parameters;
        }

        public List<int> GetTotalFee()
        {
            List<int> totalfee = new List<int>();
            totalfee.Add(50);
            totalfee.Add(100);
            totalfee.Add(150);
            totalfee.Add(200);
            return totalfee;
        }

        #endregion

        #region 微信服务商支付通知

        public string WXMINAPayNotifyFuwushang(string key)
        {
            WXResultNotify resultNotify = new WXResultNotify();
            //获取微信通知数据
            WXPayData notifyData = resultNotify.GetNotifyData(key);

            LogEvent.WebAPILoger.Info($"回调接口测试：WXMINAPayNotifyFuwushang:{JsonConvert.SerializeObject(notifyData)}");

            if (notifyData.GetValue("return_code").ToString() == "SUCCESS")
            {
                var appName = string.Empty;
                var phone = string.Empty;
                var openid = notifyData.GetValue("openid").ToString();
                var sub_openid = notifyData.GetValue("sub_openid").ToString();
                var total_fee = int.Parse(notifyData.GetValue("total_fee").ToString());
                var attach = notifyData.GetValue("attach").ToString();
                var companyID = attach.Split('-')[0];
                if (attach.Split('-').Length == 2)
                {
                    appName = attach.Split('-')[1];
                }
                if (attach.Split('-').Length == 3)
                {
                    appName = attach.Split('-')[1];
                    phone = attach.Split('-')[2];
                }
                var nickName = WechatUtility.GetNickName(sub_openid);
                var deviceCode = notifyData.GetValue("device_info").ToString();
                var stationCode = deviceCode.Split('|')[0];
                var transaction_id = notifyData.GetValue("transaction_id").ToString();
                var out_trade_no = notifyData.GetValue("out_trade_no").ToString();
                var time_end = long.Parse(notifyData.GetValue("time_end").ToString());
                var sub_mch_id = notifyData.GetValue("sub_mch_id").ToString();

                //判断是否此订单是否已经超过2分中未响应
                bool isTimeOut = IsTimeOutNotify(time_end);
                if (isTimeOut == true)
                {
                    LogEvent.WebAPILoger.Warn($"DeviceCode={deviceCode}支付信息通知超过2分钟，不予处理。transaction_id={transaction_id},total_fee = {total_fee},nickName = {nickName},time_end = {time_end}。");
                    return "<xml><return_code><![CDATA[SUCCESS]]></return_code></xml>";
                }

                if (CreateRedisOrderFuwushang(sub_mch_id, sub_openid, total_fee, nickName, deviceCode, stationCode,
                 transaction_id, out_trade_no, time_end, EnumPaymentType.小程序微信, companyID, null, appName, phone, notifyData))

                {
                    return "<xml><return_code><![CDATA[SUCCESS]]></return_code></xml>";
                }
            }

            return "";
        }

        #endregion

        #region 收到付款后的业务部分

        public bool CreateRedisOrderFuwushang(string sub_mch_id, string openid, int total_fee, string nickName, string deviceCode,
            string stationCode, string transaction_id, string out_trade_no, long time_end, EnumPaymentType paymentType, 
            string companyID, string tid = null, string appname = null, string phone = null, WXPayData notifyData = null)
        {
            string orderid = string.Empty;
            //判定反向对接
            RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve);
            bool isXiaojuDevice = reverse.HashExists(RedisKey.ReverseGunMap, deviceCode);
            if (!string.IsNullOrEmpty(phone) && openid.Length == 11)
            {
                openid = phone;
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            //保存支付信息  银联代表银联商务接入的微信小程序
            if (paymentType == EnumPaymentType.银商小程序 || paymentType == EnumPaymentType.银商云闪付 || paymentType == EnumPaymentType.银商支付宝 || paymentType == EnumPaymentType.银商微信)
            {
                orderid = out_trade_no;
                bool isExist = redis.HashExists(RedisKey.PayOrder, orderid);
                if (isExist)
                {
                    return true;
                }
                SaveUMSOrder(sub_mch_id, openid, nickName, orderid, deviceCode, transaction_id, out_trade_no, total_fee, time_end, paymentType, companyID, tid);
            }
            else if(paymentType == EnumPaymentType.小程序微信)
            {
                orderid = out_trade_no.Length >= 28 ? out_trade_no.Substring(10, 18) : out_trade_no;
                if (isXiaojuDevice)
                {
                    orderid = teldOperator.OperatorID + orderid;
                }
                bool isExist = redis.HashExists(RedisKey.PayOrder, orderid);
                if (isExist)
                {
                    return true;
                }
                SaveWXOrder(sub_mch_id, openid, nickName, orderid, deviceCode, transaction_id, out_trade_no, total_fee, time_end, paymentType, companyID, appname);
            }

            bool result = false;
            //判断是否此订单是否已经超过2分中未相应（停止充电并退款）
            bool isTimeOut = IsTimeOutNotify(time_end);
            if (isTimeOut == true)
            {
                //RefundAndStopCharge(openid, transaction_id, out_trade_no, total_fee, deviceCode, companyID, paymentType);
                LogEvent.WebAPILoger.Warn($"DeviceCode={deviceCode}支付信息通知超过2分钟，不予处理。transaction_id={transaction_id},total_fee = {total_fee},nickName = {nickName},time_end = {time_end}。");
                return true;
            }

            var oldDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
            //string prepayID = GetTemplateMessageUserID(openid, transaction_id, paymentType, redis);
            string prepayID = "prepayID";
            WXDeviceInfo newDeviceInfo = CreateCurrentDeviceInfo(openid, total_fee, nickName, deviceCode, stationCode, transaction_id, out_trade_no, time_end, paymentType, prepayID, companyID, sub_mch_id, appname, orderid);
            if (newDeviceInfo==null)
            {
                //创建DeviceInfo失败，一般可能是充电策略拿不到。退款
                LogEvent.WebAPILoger.Error($"CreateRedisOrderFuwushang:创建DeviceInfo失败，一般可能是充电策略拿不到。退款。transaction_id={transaction_id},deviceCode={deviceCode},total_fee={total_fee}.");

                switch (paymentType)
                {
                    case EnumPaymentType.小程序微信:
                        //已经充电中的桩，再次被付款，则退款
                        WXRefund wxrefund = new WXRefund();
                        wxrefund.Transaction_id = transaction_id;
                        wxrefund.Out_trade_no = out_trade_no;
                        wxrefund.PaymentType = paymentType;
                        wxrefund.Refund_fee = total_fee;
                        wxrefund.Total_fee = total_fee;
                        wxrefund.CompanyID = companyID;
                        wxrefund.DeviceCode = deviceCode;
                        wxrefund.appname = appname;
                        RefundFuwushang(wxrefund);
                        break;

                    case EnumPaymentType.银商小程序:
                        Warrant param = new Warrant();
                        param.merOrderId = out_trade_no;
                        param.mid = sub_mch_id;
                        param.PaymentType = (int)paymentType;
                        param.tid = tid;
                        param.deviceCode = deviceCode;
                        param.companyid = companyID;
                        param.refundAmount = total_fee;

                        var payLogic = new UMSPayLogic();
                        payLogic.UMSWarrantFuwushang(param);
                        break;
                    default:
                        break;
                }
                return true;
            }


            //根据回调的数据发起充电请求
            //微信和支付宝有可能多次通知
            if (oldDeviceInfo == null)
            {
                result = ChargeAndSaveOrderInfo(newDeviceInfo, redis, notifyData);
            }
            else
            {
                if (oldDeviceInfo.Time_end >= newDeviceInfo.Time_end)//同一个设备支付时间一样或比当前还早，可以认为是重复通知
                {
                    //LogEvent.WebAPILoger.Debug($"WXMINAPayNotifyFuwushang 再次通知 if(wxDeviceInfo.Time_end >= newDeviceInfo.Time_end), wxDeviceInfo.DeviceCode={newDeviceInfo.DeviceCode}, wxDeviceInfo.Time_end={oldDeviceInfo.Time_end}, newDeviceInfo.Time_end={newDeviceInfo.Time_end}, nickName={newDeviceInfo.NickName}，total_fee={newDeviceInfo.Total_fee}，transaction_id={newDeviceInfo.Transaction_id}");
                    result = true;
                }
                else if (oldDeviceInfo.Time_end < time_end)
                {
                    if (oldDeviceInfo.OrderState != EnumWXOrderState.充电中)
                    {
                        //第一次通知，枪在redis中有缓存，则覆盖上一次充电记录
                        result = ChargeAndSaveOrderInfo(newDeviceInfo, redis, notifyData);
                    }
                    else
                    {
                        ChargeLogic chargeLogic = new ChargeLogic();
                        int gunInfo = chargeLogic.GetGunStatus(newDeviceInfo.DeviceCode);
                        //市政府院内充电站设备无连接状态
                        if (gunInfo == (int)EnumGunStatus.未插枪 && newDeviceInfo.StationID == "6101120047")
                        {
                            gunInfo = (int)EnumGunStatus.连接;
                        }
                        if (gunInfo == (int)EnumGunStatus.连接)
                        {
                            result = ChargeAndSaveOrderInfo(newDeviceInfo, redis, notifyData);
                        }
                        else
                        {
                            switch (paymentType)
                            {
                                case EnumPaymentType.小程序微信:
                                    //已经充电中的桩，再次被付款，则退款
                                    WXRefund wxrefund = new WXRefund();
                                    wxrefund.Transaction_id = transaction_id;
                                    wxrefund.Out_trade_no = out_trade_no;
                                    wxrefund.PaymentType = paymentType;
                                    wxrefund.Refund_fee = total_fee;
                                    wxrefund.Total_fee = total_fee;
                                    wxrefund.CompanyID = companyID;
                                    wxrefund.DeviceCode = deviceCode;
                                    wxrefund.appname = appname;
                                    RefundFuwushang(wxrefund);
                                    result = true;
                                    LogEvent.WebAPILoger.Info($"CreateRedisOrderFuwushang:已经充电中的桩，再次被付款，退款.transaction_id={transaction_id},deviceCode={deviceCode},total_fee={total_fee}.");
                                    break;

                                case EnumPaymentType.银商小程序:
                                    Warrant param = new Warrant();
                                    param.merOrderId = out_trade_no;
                                    param.mid = sub_mch_id;
                                    param.PaymentType = (int)paymentType;
                                    param.tid = tid;
                                    param.deviceCode = deviceCode;
                                    param.companyid = companyID;
                                    param.refundAmount = total_fee;

                                    var payLogic = new UMSPayLogic();
                                    payLogic.UMSWarrantFuwushang(param);

                                    result = true;
                                    LogEvent.WebAPILoger.Info($"银联商务CreateRedisOrderFuwushang:已经充电中的桩，再次被付款，退款.transaction_id={transaction_id},deviceCode={deviceCode},total_fee={total_fee}.");
                                    break;
                                default:
                                    break;
                            }
                        }
                    }
                }
            }
            SaveOrderPayment(newDeviceInfo);
            return result;
        }

        //private void RefundAndStopCharge(string openid, string transaction_id, string out_trade_no, int total_fee, string deviceCode, string companyID, EnumPaymentType paymentType)
        //{
        //    try
        //    {
        //        RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
        //        WXRefund wxrefund = new WXRefund();
        //        wxrefund.Transaction_id = transaction_id;
        //        wxrefund.Out_trade_no = out_trade_no;
        //        wxrefund.PaymentType = paymentType;
        //        wxrefund.Refund_fee = total_fee;
        //        wxrefund.Total_fee = total_fee;
        //        wxrefund.CompanyID = companyID;
        //        wxrefund.DeviceCode = deviceCode;
        //        ChargeLogic chargeLogic = new ChargeLogic();
        //        var wxdevice = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);

        //        if (wxdevice != null && wxdevice.Openid == openid && wxdevice.Transaction_id == transaction_id)
        //        {
        //            chargeLogic.StopCharge(wxdevice);
        //        }
        //        RefundFuwushang(wxrefund);
        //    }
        //    catch (Exception e)
        //    {
        //        LogEvent.WebAPILoger.Error($"RefundAndStopCharge,transaction_id={transaction_id},deviceCode={deviceCode},total_fee={total_fee},companyID={companyID}", e);
        //    }
        //}

        /// <summary>
        /// 验证是否可以开启充电
        /// </summary>
        /// <param name="deviceID"></param>
        /// <returns></returns>
        public bool IsUsable(string deviceCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WebAPI);
            if (redis.HashExists("DeviceBlacklist", deviceCode))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        private string GetTemplateMessageUserID(string openid, string transaction_id, EnumPaymentType paymentType, RedisHelper redis)
        {
            string prepayID = string.Empty;
            switch (paymentType)
            {
                case EnumPaymentType.小程序微信:
                    prepayID = redis.HashGetString(RedisKey.WXPrepayID, openid);
                    break;
                case EnumPaymentType.银商小程序:
                    prepayID = redis.HashGetString(RedisKey.WXPrepayID, openid);
                    break;
                case EnumPaymentType.威富通微信:
                    break;
                default:
                    break;
            }

            return prepayID;
        }

        public WXDeviceInfo CreateCurrentDeviceInfo(string openid, int total_fee, string nickName, string deviceCode, string stationCode,
          string transaction_id, string out_trade_no, long time_end, EnumPaymentType paymentType, string prepayID, string companyID, 
          string sub_mch_id, string appname ,string orderid)
        {
            var deviceInfo = new WXDeviceInfo();
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Struct);
            RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve);
            string reverseDeviceCode = reverse.HashGetString(RedisKey.ReverseGunMap, deviceCode);
            var now = DateTime.Now;
            List<StrategyInfo> chargeStrategy;
            LCO_Station station = WechatUtility.GetStation(deviceCode);
            deviceInfo.StationID = station.StationId;
            deviceInfo.StationName = station.StationName;
            deviceInfo.OrderID = orderid;

            //反向对接
            if (!string.IsNullOrEmpty(reverseDeviceCode))
            {
                List<policyInfo> xiaojuPolicy = GetBusinessPolicy(deviceCode);
                if (xiaojuPolicy != null && xiaojuPolicy.Any ())
                {
                    List<ChargeStrategy> policy = ConstructChargeStrategy(xiaojuPolicy);
                    chargeStrategy = policy.SelectMany(p => p.Strategy).ToList();
                    deviceInfo.ChargeStrategy = chargeStrategy;
                    deviceInfo.ServiceFeeList = GetXiaojuFee(xiaojuPolicy, station.StationId, EnumPriceType.服务费);
                    deviceInfo.ElectricityFeeList = GetXiaojuFee(xiaojuPolicy, station.StationId, EnumPriceType.电费);
                }
                else
                {
                    //小桔获取不到充电策略 
                    LogEvent.WebAPILoger.Info($"CreateCurrentDeviceInfo:小桔设备充电策略拿不到。创建deviceinfo失败。deviceCode={deviceCode}。");
                    return null;
                }
            }
            else
            {
                //string serverDiscountKey = "";
                //if (paymentType== EnumPaymentType.微信刷卡)
                //{
                //    serverDiscountKey = string.Format("{0}|{1}", openid.Split('|').FirstOrDefault(), station.StationId);
                //}
                //chargeStrategy = GetNewChargeStrategy(station.StationId, serverDiscountKey, now);
                //deviceInfo.ChargeStrategy = chargeStrategy;
                //deviceInfo.ServiceFeeList = GetNewServiceFee(station.StationId);
                //deviceInfo.ElectricityFeeList = GetNewElectricityFee(station.StationId, now);
                string serverDiscountKey = "";
                if (paymentType == EnumPaymentType.微信刷卡)
                {
                    serverDiscountKey = openid.Split('|').FirstOrDefault();
                }
                //从接口获取充电电费和服务费
                Strategy effectiveFee = GetEffectiveFee(station.StationId, now, paymentType, serverDiscountKey);
                if (effectiveFee != null && effectiveFee.Strategies.Any())
                {
                    deviceInfo.ChargeStrategy = effectiveFee.Strategies;
                    deviceInfo.ServiceFeeList = effectiveFee.ServiceFees;
                    deviceInfo.ElectricityFeeList = effectiveFee.ElectricFees;
                }
                else
                {
                    //获取不到充电策略 
                    LogEvent.WebAPILoger.Info($"CreateCurrentDeviceInfo:设备充电策略拿不到。创建deviceinfo失败。deviceCode={deviceCode}。");
                    return null;
                }
            }

            deviceInfo.VIP = 0;
            deviceInfo.Openid = openid;
            deviceInfo.appname = appname;
            //deviceInfo.DeviceType = int.Parse(deviceCode.Split('|')[1]) > 180 ? EnumSubDeviceType.直流充电枪 : EnumSubDeviceType.交流充电枪;//直流canID从181到230
            var deviceTemp = redis.HashGet<cfg_device>(RedisKey.DeviceByDeviceCode, deviceCode);
            deviceInfo.DeviceType = deviceTemp == null ? 0 : deviceTemp.subdevicetype;
            deviceInfo.StationCode = stationCode; // 实际上是集控ID
            deviceInfo.DeviceCode = deviceCode;
            deviceInfo.NickName = nickName;
            deviceInfo.OrderState = EnumWXOrderState.充电中;
            deviceInfo.GunState = EnumGunStatus.启动中;
            deviceInfo.Total_fee = total_fee;
            deviceInfo.RemainingFee = total_fee * 10;

            switch (paymentType)
            {
                case EnumPaymentType.小程序微信:
                    deviceInfo.Transaction_id = transaction_id;
                    deviceInfo.Out_trade_no = out_trade_no;
                    break;
                case EnumPaymentType.银商云闪付:
                case EnumPaymentType.银商小程序:
                case EnumPaymentType.银商支付宝:
                case EnumPaymentType.银商微信:
                    deviceInfo.tid = sub_mch_id;
                    deviceInfo.targetOrderId = transaction_id;
                    deviceInfo.merOrderId = out_trade_no;
                    break;
                default:
                    break;
            }

            deviceInfo.StartTime = now.ToString("yyyy-MM-dd HH:mm:ss");
            // deviceInfo.ServiceFee = servicePrice;
            deviceInfo.Time_end = time_end;
            deviceInfo.PaymentType = paymentType;
            deviceInfo.Voltage = "";
            deviceInfo.Current = "";
            deviceInfo.Power = "";
            deviceInfo.Duration = "";
            deviceInfo.PrepayID = prepayID;
            deviceInfo.CompanyID = companyID;
            deviceInfo.ForwardPower = -0.001; // 初始电表读数为负数，方便区分是电表初始读数真是0，还是没有采集上来呢

            deviceInfo.ChargePower = new List<StrategyPowerConsumption>();
            foreach (var item in deviceInfo.ChargeStrategy)
            {
                var strategyPowerConsumption = new StrategyPowerConsumption();
                strategyPowerConsumption.StartTime = item.StartTime;
                strategyPowerConsumption.EndTime = item.EndTime;
                strategyPowerConsumption.PowerConsumption = 0;
                deviceInfo.ChargePower.Add(strategyPowerConsumption);
            }

            return deviceInfo;
        }

        private List<Price> GetXiaojuFee(List<policyInfo> xiaojuPolicy,string stationid, EnumPriceType priceType)
        {
            List<Price> serviceFeeStrategyList = new List<Price>();

            xiaojuPolicy = xiaojuPolicy.OrderBy(p => p.StartTime).ToList();
            for (int i = 0; i < xiaojuPolicy.Count; i++)
            {
                string starttime = xiaojuPolicy[i].StartTime;
                string endtime = (i == (xiaojuPolicy.Count - 1)) ? "235959" : xiaojuPolicy[i + 1].StartTime;
                Price price = new Price();
                //格式“HHmmss”,6 字符
                price.startTime = new TimeSpan(int.Parse(starttime.Substring(0, 2)), int.Parse(starttime.Substring(2, 2)), int.Parse(starttime.Substring(4, 2)));
                price.endTime = new TimeSpan(int.Parse(endtime.Substring(0, 2)), int.Parse(endtime.Substring(2, 2)), int.Parse(endtime.Substring(4, 2)));
                if (priceType==EnumPriceType.服务费)
                {
                    price.price = xiaojuPolicy[i].ServicePrice;
                }
                else if (priceType == EnumPriceType.电费)
                {
                    price.price = xiaojuPolicy[i].ElecPrice;
                }
                else
                {
                    price.price = 0;
                }
                price.stationid = stationid;
                price.useTime = DateTime.Now.AddDays(-1);

                serviceFeeStrategyList.Add(price);
            }

            return serviceFeeStrategyList;
        }

        private List<StrategyInfo> GetNewChargeStrategy(string stationID, string discountKey, DateTime now)
        {
            List<StrategyInfo> chargeStrategyList = null;
            ChargeStrategy nowChargeStrategy = null;
            List<ChargeStrategy> chargeStrategy = null;

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Struct);

            if (!string.IsNullOrEmpty(discountKey))
            {
                //先看有没有打折的活动
                chargeStrategy = redis.HashGet<List<ChargeStrategy>>(RedisKey.ChargeDiscountStrategy, discountKey);
                if (chargeStrategy != null && chargeStrategy.Any())
                {
                    nowChargeStrategy = chargeStrategy.FirstOrDefault(p => p.UseTime <= now && p.EndUseTime > now);
                }
            }

            //没有打折活动 或者 活动不在有效期内，重新获取场站统一充电策略
            if (nowChargeStrategy == null)
            {
                if (!string.IsNullOrEmpty(stationID))
                {
                    chargeStrategy = redis.HashGet<List<ChargeStrategy>>(RedisKey.NewChargeStrategy, stationID);
                    if (chargeStrategy != null && chargeStrategy.Any())
                    {
                        nowChargeStrategy = chargeStrategy.FirstOrDefault(p => p.UseTime <= now && p.EndUseTime > now);
                    }
                }

                //场站统一充电策略也没有有效期内的，给一个默认值
                if (nowChargeStrategy == null)
                {
                    var chargeStrategyToString = chargeStrategy == null ? "null" : JsonConvert.SerializeObject(chargeStrategy);
                    LogEvent.CMUServerLoger.Error($"stationID={stationID}，没有获取到有效的充电策略，策略={chargeStrategyToString}。");

                    nowChargeStrategy = new ChargeStrategy();
                    nowChargeStrategy.Strategy = new List<StrategyInfo>();
                }
            }

            chargeStrategyList = nowChargeStrategy.Strategy;

            return chargeStrategyList;
        }

        private List<Price> GetNewElectricityFee(string stationID, DateTime now)
        {
            List<Price> electricityFeeStrategyList = null;
            PriceStrategy nowElectricityFeeStrategy = null;
            List<PriceStrategy> electricityFeeStrategy = null;
            if (!string.IsNullOrEmpty(stationID))
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WebAPI);
                electricityFeeStrategy = redis.HashGet<List<PriceStrategy>>(RedisKey.NewElectricFee, stationID);

                if (electricityFeeStrategy != null && electricityFeeStrategy.Any())
                {
                    nowElectricityFeeStrategy = electricityFeeStrategy.FirstOrDefault(p => p.UseTime <= now && p.EndUseTime > now);
                }

            }

            if (nowElectricityFeeStrategy == null)
            {
                var electricityFeeStrategyToString = electricityFeeStrategy == null ? "null" : JsonConvert.SerializeObject(electricityFeeStrategy);
                LogEvent.CMUServerLoger.Error($"stationID={stationID}，没有获取到有效的电费策略，策略={electricityFeeStrategyToString}。");

                electricityFeeStrategyList = new List<Price>();
                //var price = new Price();
                //price.startTime = new TimeSpan(0, 0, 0);
                //price.endTime = new TimeSpan(23, 59, 59);
                //price.price = 10;//默认一度电10元
                //electricityFeeStrategyList.Add(price);
            }
            else
            {
                electricityFeeStrategyList = nowElectricityFeeStrategy.Strategy;
            }

            return electricityFeeStrategyList;
        }

        public List<Price> GetNewServiceFee(string stationID)
        {
            List<Price> serviceFeeStrategyList = null;
            PriceStrategy nowServiceFeeStrategy = null;
            List<PriceStrategy> serviceFeeStrategy = null;

            if (!string.IsNullOrEmpty(stationID))
            {
                var now = DateTime.Now;
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WebAPI);
                serviceFeeStrategy = redis.HashGet<List<PriceStrategy>>(RedisKey.NewServiceFee, stationID);
                if (serviceFeeStrategy != null && serviceFeeStrategy.Any())
                {
                    nowServiceFeeStrategy = serviceFeeStrategy.FirstOrDefault(p => p.UseTime <= now && p.EndUseTime > now);
                }
            }

            if (nowServiceFeeStrategy == null)
            {
                var serviceFeeStrategyToString = serviceFeeStrategy == null ? "null" : JsonConvert.SerializeObject(serviceFeeStrategy);
                LogEvent.CMUServerLoger.Error($"stationID={stationID}，没有获取到有效的服务费策略，策略={serviceFeeStrategyToString}。");

                serviceFeeStrategyList = new List<Price>();
                //var price = new Price();
                //price.startTime = new TimeSpan(0, 0, 0);
                //price.endTime = new TimeSpan(23, 59, 59);
                //price.price = 10;//默认一度电10元
                //serviceFeeStrategyList.Add(price);
            }
            else
            {
                serviceFeeStrategyList = nowServiceFeeStrategy.Strategy;
            }

            return serviceFeeStrategyList;
        }

        private bool ChargeAndSaveOrderInfo(WXDeviceInfo newDeviceInfo, RedisHelper redis, WXPayData notifyData = null)
        {
            if (notifyData != null)
            {
                Task.Run(() =>
                {
                    RedisHelper redis15 = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                    redis15.ListRightPush("WXPayNotify", notifyData.ToJson());
                });
            }

            bool chargeResult = false;

            RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve);
            bool isXiaojuDevice = reverse.HashExists(RedisKey.ReverseGunMap, newDeviceInfo.DeviceCode);
            if (isXiaojuDevice==true)
            {
                //小桔设备启动流程
                chargeResult = XiaojuDeviceStartCharge(newDeviceInfo.DeviceCode, newDeviceInfo.OrderID);
                if (chargeResult == true)
                {
                    LogEvent.WebAPILoger.Info($"反向对接启动充电完成，reviceCode={newDeviceInfo.DeviceCode}");
                }
                else
                {
                    //小桔启动不成功，退款
                    switch (newDeviceInfo.PaymentType)
                    {
                        case EnumPaymentType.小程序微信:
                            WXRefund wxrefund = new WXRefund();
                            wxrefund.Transaction_id = newDeviceInfo.Transaction_id;
                            wxrefund.Out_trade_no = newDeviceInfo.Out_trade_no;
                            wxrefund.PaymentType = newDeviceInfo.PaymentType;
                            wxrefund.Refund_fee = newDeviceInfo.Total_fee;
                            wxrefund.Total_fee = newDeviceInfo.Total_fee;
                            wxrefund.CompanyID = newDeviceInfo.CompanyID;
                            wxrefund.DeviceCode = newDeviceInfo.DeviceCode;
                            wxrefund.appname = newDeviceInfo.appname;
                            RefundFuwushang(wxrefund);
                            LogEvent.WebAPILoger.Warn($"ChargeAndSaveOrderInfo:小桔设备启动失败，具体原因参考ChargeStart日志文件。退款。transaction_id={wxrefund.Transaction_id},deviceCode={wxrefund.DeviceCode},total_fee={wxrefund.Total_fee}.");
                            break;

                        
                        case EnumPaymentType.银商小程序:
                            Warrant umsefund = new Warrant();
                            umsefund.merOrderId = newDeviceInfo.merOrderId;
                            umsefund.refundAmount = newDeviceInfo.Total_fee;
                            umsefund.mid = newDeviceInfo.tid;
                            umsefund.tid = "00000001"; //历史遗留原因，deviceinfo中tid字段放的是mid的值
                            umsefund.deviceCode = newDeviceInfo.DeviceCode;
                            umsefund.companyid = newDeviceInfo.CompanyID;
                            umsefund.PaymentType = (int)newDeviceInfo.PaymentType;
                            var payLogic = new UMSPayLogic();
                            payLogic.UMSWarrantFuwushang(umsefund);
                            LogEvent.WebAPILoger.Info($"ChargeAndSaveOrderInfo:小桔设备启动失败，具体原因参考ChargeStart日志文件。退款。deviceCode={newDeviceInfo.DeviceCode},total_fee={newDeviceInfo.Total_fee}.");
                            break;

                        case EnumPaymentType.银商支付宝:
                        case EnumPaymentType.银商云闪付:
                        case EnumPaymentType.银商微信:
                            break;

                        case EnumPaymentType.微信刷卡:
                            break;
                        default:
                            break;
                    }

                    return true;
                }
            }

            else
            {
                //正常流程
                ChargeLogic chargeLogic = new ChargeLogic();
                int gunInfo = chargeLogic.GetGunStatus(newDeviceInfo.DeviceCode);

                //市政府院内充电站设备无连接状态
                if (gunInfo == (int)EnumGunStatus.未插枪 && newDeviceInfo.StationID=="6101120047")
                {
                    gunInfo = (int)EnumGunStatus.连接;
                }

                if (gunInfo == (int)EnumGunStatus.连接 && IsUsable(newDeviceInfo.DeviceCode) == true)
                {
                    chargeResult = Charge(newDeviceInfo);
                    LogEvent.WebAPILoger.Info($"启动充电完成，chargeResult={chargeResult}！reviceCode={newDeviceInfo.DeviceCode}");
                    Log.Info($"启动充电完成。，chargeResult={chargeResult}！reviceCode={newDeviceInfo.DeviceCode}", "success");

                }
                else
                {
                    LogEvent.WebAPILoger.Warn($"枪的状态非连接，无法启动充电！deviceCode={newDeviceInfo.DeviceCode},transation_id={newDeviceInfo.Transaction_id},refundFee={newDeviceInfo.RefundFee},gunstate={gunInfo} 或者是黑名单");
                }
            }

            if (chargeResult == true)
            {
                SaveOrderInfo(newDeviceInfo, redis);
                SendPaymentType(newDeviceInfo);
                SaveChargeingDevice(newDeviceInfo.DeviceCode);
                SaveActiveOrderRedis(newDeviceInfo);
                switch (newDeviceInfo.PaymentType)
                {
                    case EnumPaymentType.小程序微信:
                        LogEvent.WebAPILoger.Info($"微信 PayNotify startChargeResult=真,deviceCode={newDeviceInfo.DeviceCode},nickName={newDeviceInfo.NickName},total_fee={newDeviceInfo.Total_fee},transaction_id={newDeviceInfo.Transaction_id},out_trade_no={newDeviceInfo.Out_trade_no},OrderID={newDeviceInfo.OrderID}");
                        break;
                    case EnumPaymentType.银商小程序:
                    case EnumPaymentType.银商微信:
                    case EnumPaymentType.银商支付宝:
                    case EnumPaymentType.银商云闪付:
                        LogEvent.WebAPILoger.Info($"银商 PayNotify startChargeResult=真,deviceCode={newDeviceInfo.DeviceCode},nickName={newDeviceInfo.NickName},total_fee={newDeviceInfo.Total_fee},transaction_id={newDeviceInfo.merOrderId},out_trade_no={newDeviceInfo.targetOrderId},OrderID={newDeviceInfo.OrderID}");
                        break;
                    default:
                        break;
                }
                return true;
            }
            return false;
        }


        private void SaveActiveOrderRedis(WXDeviceInfo deviceinfo)
        {
            var activeOrder = new ActiveOrder();
            activeOrder.orderid = deviceinfo.OrderID;
            activeOrder.starttime = DateTime.Parse(deviceinfo.StartTime);
            activeOrder.devicecode = deviceinfo.DeviceCode;
            activeOrder.userid = deviceinfo.Openid;
            activeOrder.stationcode = deviceinfo.StationID;
            RedisHelper reddis = new RedisHelper((int)EnumUserRedisNum.RealData);

            reddis.HashSet(RedisKey.ActiveOrder, deviceinfo.DeviceCode, activeOrder);
        }


        public void SaveChargeingDevice(string deviceCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);

            var chargingdevice = redis.HashGet<List<string>>(RedisKey.SystemChargingDevice, "1");
            if (chargingdevice == null )
            {
                chargingdevice = new List<string>();
            }
            if (!chargingdevice.Contains(deviceCode))
            {
                chargingdevice.Add(deviceCode);
                redis.HashSet(RedisKey.SystemChargingDevice, "1", chargingdevice);
            }
        }

        private void SaveWXOrder(string mch_id, string openid, string nickname, string orderid, string devicecode,
            string transaction_id, string out_trade_no, int totalfee, long time_end, EnumPaymentType paymentType, string companyID, string appname)
        {
            string stationid = "";

            LCO_Station station = WechatUtility.GetStation(devicecode);
            if (station != null)
            {
                stationid = station.StationId;
            }

            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var sql = @"INSERT INTO dbo.lco_wxorder (mch_id, stationid, openid, nickname, orderid, devicecode, transaction_id, out_trade_no, totalfee, refundfee, time_end,appname) 
                        VALUES (@mch_id, @stationid, @openid, @nickname, @orderid, @devicecode, @transaction_id, @out_trade_no, @totalfee, @refundfee, @time_end,@appname) ";
                try
                {
                    //判断此订单号是否已经存在
                    var isExistPayRecordSQL = "SELECT 1 FROM lco_wxorder WHERE transaction_id=@transaction_id";
                    DynamicParameters isExistPayRecordSQLParamParam = new DynamicParameters();
                    isExistPayRecordSQLParamParam.Add("@transaction_id", transaction_id);
                    dynamic isExistPayRecord = SqlMapper.QueryFirstOrDefault(connection, isExistPayRecordSQL, isExistPayRecordSQLParamParam);
                    if (isExistPayRecord != null)
                    {
                        return;
                    }

                    DynamicParameters param = new DynamicParameters();
                    param.Add("@mch_id", mch_id);
                    param.Add("@stationid", stationid);
                    param.Add("@openid", openid);
                    param.Add("@nickname", nickname);
                    param.Add("@orderid", orderid);
                    param.Add("@devicecode", devicecode);
                    param.Add("@transaction_id", transaction_id);
                    param.Add("@out_trade_no", out_trade_no);
                    param.Add("@totalfee", totalfee);
                    param.Add("@refundfee", null);
                    param.Add("@time_end", time_end);
                    param.Add("@appname", appname);
                    IDbCommand cmd = connection.CreateCommand();
                    cmd.CommandTimeout = 180;

                    int affectedRowNum = SqlMapper.Execute(connection, sql, param);
                    if (affectedRowNum != 1)
                    {
                        Log.Info("", $"SaveWXOrder插入失败。affectedRowNum={affectedRowNum},orderid={orderid},devicecode={devicecode},total_fee={totalfee},transaction_id={transaction_id},out_trade_no={out_trade_no},time_end={time_end}。");
                        throw new Exception("SaveWXOrder Exception with param");
                    }
                    else
                    {
                        RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                        bool isExist = redis.HashExists(RedisKey.PayOrder, orderid);
                        if (!isExist)
                        {
                            var umsOrder = new PayOrder
                            {
                                orderid = orderid,
                                devicecode = devicecode,
                                transaction_id = transaction_id,
                                out_trade_no = out_trade_no,
                                totalfee = totalfee,
                                time_end = time_end,
                                openid = openid,
                                stationid = stationid,
                                mid = mch_id,
                                companyid = companyID,
                                paymentType = (int)paymentType,
                                nickname = nickname,
                            };
                            redis.HashSet(RedisKey.PayOrder, orderid, umsOrder);
                        }
                    }
                }
                catch (Exception)
                {
                    try
                    {
                        //判断此订单号是否已经存在
                        var isExistPayRecordSQL = "SELECT 1 FROM lco_wxorder WHERE transaction_id=@transaction_id";
                        DynamicParameters isExistPayRecordSQLParamParam = new DynamicParameters();
                        isExistPayRecordSQLParamParam.Add("@transaction_id", transaction_id);
                        dynamic isExistPayRecord = SqlMapper.QueryFirstOrDefault(connection, isExistPayRecordSQL, isExistPayRecordSQLParamParam);
                        if (isExistPayRecord != null)
                        {
                            return;
                        }

                        DynamicParameters param2 = new DynamicParameters();
                        param2.Add("@mch_id", mch_id);
                        param2.Add("@stationid", stationid);
                        param2.Add("@openid", openid);
                        param2.Add("@nickname", "nickname");
                        param2.Add("@orderid", orderid);
                        param2.Add("@devicecode", devicecode);
                        param2.Add("@transaction_id", transaction_id);
                        param2.Add("@out_trade_no", out_trade_no);
                        param2.Add("@totalfee", totalfee);
                        param2.Add("@refundfee", null);
                        param2.Add("@time_end", time_end);
                        param2.Add("@appname", appname);
                        IDbCommand cmd = connection.CreateCommand();
                        cmd.CommandTimeout = 180;
                        int affectedRowNum = SqlMapper.Execute(connection, sql, param2);
                        if (affectedRowNum != 1)
                        {
                            LogEvent.WebAPILoger.Error($"SaveWXOrder插入失败,使用参数param2。affectedRowNum={affectedRowNum},orderid={orderid},devicecode={devicecode},total_fee={totalfee},transaction_id={transaction_id},out_trade_no={out_trade_no.Substring(10, 18)},time_end={time_end}。");
                            throw new Exception("SaveWXOrder Exception with param2");
                        }
                        else
                        {
                            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                            bool isExist = redis.HashExists(RedisKey.PayOrder, orderid);
                            if (!isExist)
                            {
                                var umsOrder = new PayOrder
                                {
                                    orderid = orderid,
                                    devicecode = devicecode,
                                    transaction_id = transaction_id,
                                    out_trade_no = out_trade_no,
                                    totalfee = totalfee,
                                    time_end = time_end,
                                    openid = openid,
                                    stationid = stationid,
                                    mid = mch_id,
                                    companyid = companyID,
                                    paymentType = (int)paymentType,
                                    nickname = nickname,
                                };
                                redis.HashSet(RedisKey.PayOrder, orderid, umsOrder);
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        LogEvent.WebAPILoger.Error($"SaveWXOrder插入异常。,orderid={orderid},devicecode={devicecode},total_fee={totalfee},transaction_id={transaction_id},out_trade_no={out_trade_no.Substring(10, 18)},time_end={time_end}。", ex);
                        //throw new Exception("SaveWXOrder Exception with param");
                    }
                }
            }

        }


        private void SaveUMSOrder(string mch_id, string openid, string nickname, string orderid, string devicecode,
           string targetOrderId, string merOrderId, int totalfee, long time_end, EnumPaymentType paymentType, string companyID, string tid)
        {

            string stationid = "";
            var station = WechatUtility.GetStation(devicecode);
            if (station!=null)
            {
                stationid = station.StationId;
            }

            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var sql = @"INSERT INTO lco_umsorder (orderid, devicecode, merOrderId, targetOrderId, totalfee, time_end, openid, stationid, mid, tid, companyid, paymentType, nickname) 
                                              VALUES (@orderid,@devicecode,@merOrderId,@targetOrderId,@totalfee,@time_end,@openid,@stationid,@mid,@tid,@companyid,@paymentType,@nickname) ";
                try
                {
                    //判断此订单号是否已经存在
                    var isExistPayRecordSQL = "SELECT 1 FROM lco_umsorder WHERE targetOrderId=@targetOrderId";
                    DynamicParameters isExistPayRecordSQLParamParam = new DynamicParameters();
                    isExistPayRecordSQLParamParam.Add("@targetOrderId", targetOrderId);
                    dynamic isExistPayRecord = SqlMapper.QueryFirstOrDefault(connection, isExistPayRecordSQL, isExistPayRecordSQLParamParam);
                    if (isExistPayRecord != null)
                    {
                        return;
                    }

                    DynamicParameters param = new DynamicParameters();
                    param.Add("@orderid", orderid);
                    param.Add("@devicecode", devicecode);
                    param.Add("@merOrderId", merOrderId);
                    param.Add("@targetOrderId", targetOrderId);
                    param.Add("@totalfee", totalfee);
                    param.Add("@time_end", time_end);
                    param.Add("@openid", openid);
                    param.Add("@stationid", stationid);
                    param.Add("@mid", mch_id);
                    param.Add("@tid", tid);
                    param.Add("@companyid", companyID);
                    param.Add("@paymentType", paymentType);
                    param.Add("@nickname", nickname);

                    IDbCommand cmd = connection.CreateCommand();
                    cmd.CommandTimeout = 180;

                    int affectedRowNum = SqlMapper.Execute(connection, sql, param);
                    if (affectedRowNum != 1)
                    {
                        LogEvent.WebAPILoger.Error($"SaveUMSOrder插入失败。mch_id = {mch_id},affectedRowNum={affectedRowNum},devicecode={devicecode},total_fee={totalfee},merOrderId={merOrderId},targetOrderId={targetOrderId},time_end={time_end}。");
                        throw new Exception("SaveWXOrder Exception with param");
                    }
                    else
                    {
                        RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                        bool isExist = redis.HashExists(RedisKey.PayOrder, orderid);
                        if (!isExist)
                        {
                            var umsOrder = new PayOrder
                            {
                                orderid = orderid,
                                devicecode = devicecode,
                                merOrderId = merOrderId,
                                targetOrderId = targetOrderId,
                                totalfee = totalfee,
                                time_end = time_end,
                                openid = openid,
                                stationid = stationid,
                                mid = mch_id,
                                tid = tid,
                                companyid = companyID,
                                paymentType = (int)paymentType,
                                nickname = nickname,
                            };
                            redis.HashSet(RedisKey.PayOrder, orderid, umsOrder);
                        }
                    }
                }
                catch (Exception)
                {
                    try
                    {
                        //判断此订单号是否已经存在
                        var isExistPayRecordSQL = "SELECT 1 FROM lco_umsorder WHERE targetOrderId=@targetOrderId";
                        DynamicParameters isExistPayRecordSQLParamParam = new DynamicParameters();
                        isExistPayRecordSQLParamParam.Add("@targetOrderId", targetOrderId);
                        dynamic isExistPayRecord = SqlMapper.QueryFirstOrDefault(connection, isExistPayRecordSQL, isExistPayRecordSQLParamParam);
                        if (isExistPayRecord != null)
                        {
                            return;
                        }

                        DynamicParameters param2 = new DynamicParameters();
                        param2.Add("@orderid", orderid);
                        param2.Add("@devicecode", devicecode);
                        param2.Add("@merOrderId", merOrderId);
                        param2.Add("@targetOrderId", targetOrderId);
                        param2.Add("@totalfee", totalfee);
                        param2.Add("@time_end", time_end);
                        param2.Add("@openid", openid);
                        param2.Add("@stationid", stationid);
                        param2.Add("@mid", mch_id);
                        param2.Add("@tid", tid);
                        param2.Add("@companyid", companyID);
                        param2.Add("@paymentType", paymentType);
                        param2.Add("@nickname", "nickname");

                        IDbCommand cmd = connection.CreateCommand();
                        cmd.CommandTimeout = 180;
                        int affectedRowNum = SqlMapper.Execute(connection, sql, param2);
                        if (affectedRowNum != 1)
                        {
                            LogEvent.WebAPILoger.Error($"SaveUMSOrder插入失败,使用参数param2。affectedRowNum={affectedRowNum},mch_id = {mch_id},devicecode={devicecode},total_fee={totalfee},merOrderId={merOrderId},targetOrderId={targetOrderId},time_end={time_end}。");
                            throw new Exception("SaveWXOrder Exception with param2");
                        }
                        else
                        {
                            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                            bool isExist = redis.HashExists(RedisKey.PayOrder, orderid);
                            if (!isExist)
                            {
                                var umsOrder = new PayOrder
                                {
                                    orderid = orderid,
                                    devicecode = devicecode,
                                    merOrderId = merOrderId,
                                    targetOrderId = targetOrderId,
                                    totalfee = totalfee,
                                    time_end = time_end,
                                    openid = openid,
                                    stationid = stationid,
                                    mid = mch_id,
                                    tid = tid,
                                    companyid = companyID,
                                    paymentType = (int)paymentType,
                                    nickname = nickname,
                                };
                                redis.HashSet(RedisKey.PayOrder, orderid, umsOrder);
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        LogEvent.WebAPILoger.Error($"SaveUMSOrder插入异常。 mch_id = {mch_id},devicecode={devicecode},total_fee={totalfee},merOrderId={merOrderId},targetOrderId={targetOrderId},time_end={time_end}。", ex);
                        //throw new Exception("SaveWXOrder Exception with param");
                    }
                }
            }

        }

        private void SendPaymentType(WXDeviceInfo wxDeviceInfo)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.HisData);
            DevicePaymentType paymentType = new DevicePaymentType();
            paymentType.DeviceCode = wxDeviceInfo.DeviceCode;
            paymentType.Openid = wxDeviceInfo.Openid;
            paymentType.OrderID = wxDeviceInfo.OrderID;
            paymentType.Overmoney = wxDeviceInfo.Total_fee;
            paymentType.PaymentType = wxDeviceInfo.PaymentType;
            paymentType.CardID = wxDeviceInfo.Openid;
            redis.HashSet(RedisKey.PaymentType, paymentType.DeviceCode, paymentType);
        }

        private bool SaveOrderInfo(WXDeviceInfo newDeviceInfo, RedisHelper redis)
        {
            redis.HashSet(RedisKey.OpenID, newDeviceInfo.Openid, newDeviceInfo.DeviceCode);//旧版本
            try
            {
                List<string> chargelist = new List<string>();
                string dataKey = "";
                //手机APP
                if (newDeviceInfo.Openid.Length == 11)
                {
                    dataKey = newDeviceInfo.Openid;
                }
                else if (newDeviceInfo.Openid.ToArray().Contains('|') && newDeviceInfo.Openid.Split('|')[1].Length == 11)
                {
                    dataKey = newDeviceInfo.Openid.Split('|')[1];
                }
                else
                {
                    dataKey = newDeviceInfo.Openid.Substring(newDeviceInfo.Openid.Length - 28, 28);
                }

                chargelist = redis.HashGet<List<string>>(RedisKey.ChargingDevice, dataKey);
                if (chargelist == null)
                {
                    chargelist = new List<string>();
                }
                if (!chargelist.Contains(newDeviceInfo.DeviceCode))
                {
                    chargelist.Add(newDeviceInfo.DeviceCode);
                }
                redis.HashSet<List<string>>(RedisKey.ChargingDevice, dataKey, chargelist);

            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"SaveOrderInfo 写入chargingDevice fail. Openid={newDeviceInfo.Openid}, deviceCode={newDeviceInfo.DeviceCode}", e);
            }
            redis.HashSet(RedisKey.DeviceInfo, newDeviceInfo.DeviceCode, newDeviceInfo);
            return true;
        }

        private bool Charge(WXDeviceInfo newDeviceInfo)
        {
            //启动充电
            ChargeApplyInfo apply = GetStartChargeApply(newDeviceInfo);
            var startChargeResult = StartCharge(apply);

            return startChargeResult;
        }

        private ChargeApplyInfo GetStartChargeApply(WXDeviceInfo newDeviceInfo)
        {
            var apply = new ChargeApplyInfo();
            apply.MarkID = newDeviceInfo.OrderID;
            apply.StationCode = newDeviceInfo.StationCode;
            apply.DeviceCode = newDeviceInfo.DeviceCode;
            apply.ControlID = 5001;
            apply.UserID = newDeviceInfo.Openid;
            apply.UserName = newDeviceInfo.NickName;
            apply.OrderID = newDeviceInfo.OrderID;
            apply.ChargeType = (int)EnumChargeType.立即充电;
            apply.Value = 0;
            apply.ValueDes = "";
            apply.Arrearage = 0;
            //设置金额 微信中的金额单位是分，协议中是厘
            apply.CommonBalance = newDeviceInfo.Total_fee * 10;
            //获取充电策略
            apply.ChargeList = newDeviceInfo.ChargeStrategy;
            apply.ChargeCount = newDeviceInfo.ChargeStrategy.Count;
            apply.DischargeList = new List<StrategyInfo>();
            apply.DischargeCount = apply.DischargeList.Count;

            return apply;
        }

        private bool StartCharge(ChargeApplyInfo apply)
        {
            var uri = ControlAPI + "StartCharge";
            LogEvent.WebAPILoger.Info($"{apply.DeviceCode} StartCharge 开始充电指令");
            var result = SendControl(apply, uri);

            return result;
        }

        private bool SendControl(object postData, string uri)
        {
            bool result = false;

            var apiResult = "";
            using (var httpClient = new HttpClient())
            {
                string jsonString = JsonConvert.SerializeObject(postData);
                Log.Info("SendControl jsonString = " + jsonString, "");
                LogEvent.WebAPILoger.Debug("SendControl jsonString = " + jsonString);
                byte[] bytes = System.Text.Encoding.UTF8.GetBytes(jsonString);
                using (StreamContent sc = new StreamContent(new MemoryStream(bytes)))
                {
                    sc.Headers.ContentLength = bytes.Length;
                    sc.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                    var httpResponseMessage = httpClient.PostAsync(uri, sc).Result;
                    //if (httpResponseMessage.IsSuccessStatusCode)
                    //{
                    //    apiResult = httpResponseMessage.Content.ReadAsStringAsync().Result;
                    //}
                    apiResult = httpResponseMessage.Content.ReadAsStringAsync().Result;
                    LogEvent.WebAPILoger.Debug("SendControl apiResult = " + apiResult);
                }
            }

            var jsonResult = JObject.Parse(apiResult);
            if (jsonResult.GetValue("code").ToString() == ((int)EnumAPIResult.Succeed).ToString())
            {
                LogEvent.WebAPILoger.Info("SendControl SUCCESS");
                result = true;
            }
            else
            {
                LogEvent.WebAPILoger.Error("SendControl ERROR, error message = " + jsonResult.GetValue("error").ToString(Formatting.None));
            }

            return result;
        }
        /// <summary>
        /// 判断是否超过了通知时效
        /// </summary>
        /// <param name="time_end"></param>
        /// <returns></returns>
        public bool IsTimeOutNotify(long wxtime_end)
        {
            var now = DateTime.Now;
            var result = false;
            try
            {
                DateTime s = DateTime.ParseExact(wxtime_end.ToString(), "yyyyMMddHHmmss", null);
                if (now.Subtract(s).TotalMinutes > 2)
                {
                    result = true;
                }

            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error("判断是否超过了通知时效出错", e);
            }
            return result;
        }

        #endregion

        #region 共用

        /// <summary>
        /// 生成时间戳，标准北京时间，时区为东八区，自1970年1月1日 0点0分0秒以来的秒数
        /// </summary>
        /// <returns></returns>
        public string GenerateTimeStamp()
        {
            TimeSpan ts = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, 0);
            return Convert.ToInt64(ts.TotalSeconds).ToString();
        }

        /// <summary>
        /// 生成随机串，随机串包含字母或数字
        /// </summary>
        /// <returns></returns>
        public string GenerateNonceStr()
        {
            return Guid.NewGuid().ToString("N");
        }

        private string GenerateOutTradeNo(string sub_MCHID)
        {
            var orderID = GenerateOrderID();
            return string.Format("{0}{1}", sub_MCHID, orderID);
        }

        private string GenerateOrderID()
        {
            System.Security.Cryptography.RNGCryptoServiceProvider csp = new System.Security.Cryptography.RNGCryptoServiceProvider();
            byte[] byteCsp = new byte[3];
            csp.GetBytes(byteCsp);
            return string.Format("{0}{1}", DateTime.Now.ToString("yyMMddHHmmss"), BitConverter.ToString(byteCsp).Replace("-", ""));
        }

        public WXSubMCHInfo GetSubAPPIDandMCHID(string companyID)
        {
            if (string.IsNullOrEmpty(companyID))
            {
                LogEvent.WebAPILoger.Error($"GetSubAPPIDandMCHID companyID长度不足,companyID={companyID}。");
                throw new Exception("companyID长度不足。");
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var wxSubMCHInfo = redis.HashGet<WXSubMCHInfo>(RedisKey.SubMCH, companyID);

            if (wxSubMCHInfo == null || string.IsNullOrEmpty(wxSubMCHInfo.MCHID))
            {
                LogEvent.WebAPILoger.Error($"GetSubAPPIDandMCHID companyID={companyID},子商户信息不存在");
            }

            return wxSubMCHInfo;
        }

        #endregion

        #region 微信支付通知测试

        public string WXMINAPayNotifyTest()
        {
            WXResultNotify resultNotify = new WXResultNotify();
            //获取微信通知数据
            WXPayData notifyData = resultNotify.GetNotifyData(WxPayConfig.KEY);

            if (notifyData.GetValue("return_code").ToString() == "SUCCESS")
            {
                var openid = notifyData.GetValue("openid").ToString();
                var sub_openid = notifyData.GetValue("sub_openid").ToString();
                var total_fee = int.Parse(notifyData.GetValue("total_fee").ToString());
                var attach = notifyData.GetValue("attach").ToString();
                var companyID = attach.Split('|')[0];
                var nickName = attach.Split('|')[1];
                var deviceCode = notifyData.GetValue("device_info").ToString();
                var stationCode = deviceCode.Split('|')[0];
                var transaction_id = notifyData.GetValue("transaction_id").ToString();
                var out_trade_no = notifyData.GetValue("out_trade_no").ToString();
                var time_end = long.Parse(notifyData.GetValue("time_end").ToString());

                LogEvent.NotificationLoger.Debug($"notifyData.ToXml()={notifyData.ToXml()}");

                return "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>";
            }

            return "";
        }

        #endregion

        #region 微信-卡绑定

        /// <summary>
        /// 获取卡信息
        /// </summary>
        /// <param name="userToken"></param>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        public IEnumerable<CardInfo> GetCardInfoByApplet(string userToken, string deviceCode)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            RedisHelper redis6 = new RedisHelper((int)EnumUserRedisNum.HisData);
            StringBuilder sb = new StringBuilder();
            var cardList = new List<CardInfo>();
            List<string> usableCard = new List<string>();

            var openid = WechatUtility.GetOpenIDByUserToken(userToken);
            if (string.IsNullOrEmpty(openid))
            {
                if (string.IsNullOrEmpty(openid))
                {
                    throw new Exception("Token过期，请重新登陆");
                }
            }
            string phone = redis.HashGetString(RedisKey.UserAndPhone, openid) == null ? string.Empty : redis.HashGetString(RedisKey.UserAndPhone, openid);
            if (openid.Length == 11)
            {
                phone = openid;
            }

            try
            {
                if (phone != null)
                {
                    using (IDbConnection conn = WechatUtility.CreateDBConnection())
                    {
                        string sql = $"SELECT A.overmoney/100.00 as overMoney,A.cardid,A.frozenMoney/100.00 AS frozenMoney,C.name AS companyname FROM cfg_card A,lco_customer B,lco_manufacturer C WHERE A.customerid = b.customerid AND B.phone= '{phone}' AND B.deleteOrUser = 0 AND A.type = 1 AND A.OneMore =1 AND substring(A.cardid,1,9) = C.cardInfo ";
                        cardList = SqlMapper.Query<CardInfo>(conn, sql).ToList();

                        string usablePhone = $@"SELECT cardid FROM cfg_card_usablephone WHERE usablePhone LIKE '%{phone}%'";
                        usableCard = SqlMapper.Query<string>(conn, usablePhone).ToList();
                        if (usableCard.Count > 0)
                        {
                            foreach (var item in usableCard)
                            {
                                sb.Append("'");
                                sb.Append(item);
                                sb.Append("',");
                            }
                            sb.Remove(sb.Length - 1, 1);
                            string cardinfosql = $"SELECT A.overmoney/100.00 as overMoney,A.cardid,A.frozenMoney/100.00 AS frozenMoney,C.name AS companyname FROM cfg_card A,lco_customer B,lco_manufacturer C WHERE A.customerid = b.customerid AND    B.deleteOrUser = 0 AND A.type = 1 AND A.OneMore =1 AND substring(A.cardid,1,9) = C.cardInfo AND A.cardid IN ({sb})";
                            cardList.AddRange(SqlMapper.Query<CardInfo>(conn, cardinfosql).ToList());
                        }

                    }
                    string stationid = WechatUtility.GetStationId(deviceCode);
                    string companyid = redis.HashGet<LCO_Station>(RedisKey.Station, stationid).CompanyID;
                    cardList.RemoveAll(d => d.cardid.Substring(0, 9) != companyid);


                    foreach (var item in cardList)
                    {
                        if (item.cardid.Substring(0, 10) != companyid)
                        {
                            continue;
                        }
                        string frozen = redis6.HashGetString(RedisKey.FrozenMoney, item.cardid);
                        if (frozen != null)
                        {
                            item.frozenMoney = Double.Parse(frozen) + item.frozenMoney;
                        }
                    }
                }

            }
            catch (Exception)
            {
                LogEvent.WebAPILoger.Error($"GetCardInfoByApplet phone={phone} 获取卡信息异常");
            }
            return cardList;
        }

        /// <summary>
        /// 申请卡
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <param name="userToken"></param>
        /// <param name="name"></param>
        /// <param name="carPlate"></param>
        /// <returns></returns>
        public bool ApplyCard(CardOfApply applycard)
        {
            bool result = false;
            if (applycard == null)
            {
                return result;
            }
            RedisHelper redis15 = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            string stationid = WechatUtility.GetStationId(applycard.deviceCode);
            LCO_Station station = redis15.HashGet<LCO_Station>(RedisKey.Station, stationid);
            var openid = WechatUtility.GetOpenIDByUserToken(applycard.userToken);
            if (string.IsNullOrEmpty(openid) && station == null)
            {
                return result;
            }
            string phone = redis15.HashGetString(RedisKey.UserAndPhone, openid) == null ? string.Empty : redis15.HashGetString(RedisKey.UserAndPhone, openid);
            if (phone != null)
            {
                string companyid = station.CompanyID;
                result = insertIntoDB(companyid, phone, applycard.deviceCode, applycard.name, applycard.carPlate, applycard.cardType);
            }


            return result;
        }

        public bool insertIntoDB(string companyid, string phone, string deviceCode, string name, string carPlate, string cardType)
        {
            bool result = false;
            try
            {
                using (IDbConnection connection = WechatUtility.CreateDBConnection())
                {
                    connection.Open();

                    string seleteMaxCardidSql = $"SELECT max(cardid) FROM cfg_card WHERE cardid LIKE  '%WX'";
                    var cardNum = SqlMapper.Query<string>(connection, seleteMaxCardidSql).FirstOrDefault();
                    var a = cardNum.Substring(0, cardNum.Length - 2);
                    string cardid = Convert.ToInt64(cardNum.Substring(0, cardNum.Length - 2)) + 1 + cardType;
                    string customerid = WechatUtility.GetStationId(deviceCode) + phone;
                    string customername = name + "  " + carPlate;
                    using (IDbTransaction trans = connection.BeginTransaction())
                    {
                        try
                        {
                            var command = connection.CreateCommand();
                            command.Transaction = trans;

                            string insertCardSql = $"INSERT INTO cfg_card(cardid, customerid, overmoney, createtime, paymodel, type, password, discount, OneMore, frozenMoney) values('{cardid}','{customerid}',0,'{DateTime.Now}',1,1,123456,1,1,0)";
                            string insertCustomerSql = $"INSERT INTO lco_customer (customerid,customername,phone,email,idnumber,birthday,credit,type,deleteOrUser,money,frozenMoney) VALUES('{customerid}','{customername}','{phone}',null,'{DateTimeToStamp(DateTime.Now)}',null,0,'{companyid}',0,0,0)";

                            command.CommandText = insertCardSql;
                            command.ExecuteNonQuery();

                            command.CommandText = insertCustomerSql;
                            command.ExecuteNonQuery();


                            trans.Commit();
                        }
                        catch (Exception e)
                        {
                            trans.Rollback();
                            LogEvent.WebAPILoger.Error($"insertIntoDB baicunshi异常", e);
                        }
                        finally
                        {
                            connection.Close();
                        }

                    }

                }
                result = true;
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"insertIntoDB baicunshi异常", e);
            }

            return result;
        }

        /// <summary>
        /// 转化为时间戳
        /// </summary>
        /// <param name="now"></param>
        /// <returns></returns>
        public string DateTimeToStamp(DateTime now)
        {
            DateTime startTime = TimeZone.CurrentTimeZone.ToLocalTime(new DateTime(1970, 1, 1)); // 当地时区
            long timeStamp = (long)(now - startTime).TotalMilliseconds; // 相差毫秒数
            return timeStamp.ToString();
        }

        /// <summary>
        /// 充值下单
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        public string CardPayFuwushang(cardPayParam param)
        {

            string openid = WechatUtility.GetOpenIDByUserToken(param.userToken);
            if (string.IsNullOrEmpty(openid))
            {
                throw new ArgumentException($"param.userToken={param.userToken},此token获取不到openid");
            }


            // 保存openid和昵称的对应关系，昵称在微信支付中attch传输的话，经常因为非utf-8字符导致下单或回调失败
            SaveIDandNickName(openid, param.nickname);
            param.nickname = "nickName";
            // string companyID = "";
            // 根据设备确定此设备的费用归属哪家子商户

            WXSubMCHInfo wxSubMCHInfo = GetSubAPPIDandMCHID(param.cardid.Substring(0, 9));
            if (wxSubMCHInfo.MINA_APPID != param.APPID)
            {
                if (param.APPID == WxPayAPI.WxPayConfig.MINA_APPID)
                {
                    wxSubMCHInfo.MINA_APPSECRET = WxPayAPI.WxPayConfig.MINA_APPSECRET;
                    wxSubMCHInfo.MINA_APPID = WxPayAPI.WxPayConfig.MINA_APPID;
                }
                else
                {
                    LogEvent.WebAPILoger.Error($"此设备不属于此程序对应设备！param={param.ToString()}");
                    return "";
                }
            }

            // 统一下单
            WXPayData payData = GetUnifiedCardResultFuwushang(openid, param, wxSubMCHInfo, param.cardid.Substring(0, 9));

            // 保存预付款ID，为了主动推送模板消息
            SavePrepayID(openid, payData.GetValue("prepay_id").ToString());

            var parameters = GetJsApiParametersFuwushang(payData, wxSubMCHInfo, "");

            return parameters;

        }

        public string WXCardPayNotifyFuwushang()
        {
            WXResultNotify resultNotify = new WXResultNotify();
            //获取微信通知数据
            WXPayData notifyData = resultNotify.GetNotifyData(WxPayConfig.KEY);

            if (notifyData.GetValue("return_code").ToString() == "SUCCESS")
            {
                var openid = notifyData.GetValue("openid").ToString();
                var sub_openid = notifyData.GetValue("sub_openid").ToString();
                var total_fee = int.Parse(notifyData.GetValue("total_fee").ToString());
                var attach = notifyData.GetValue("attach").ToString();
                var companyID = attach.Split('|')[0];
                var nickName = WechatUtility.GetNickName(sub_openid);
                var cardid = notifyData.GetValue("device_info").ToString();

                var transaction_id = notifyData.GetValue("transaction_id").ToString();
                var sub_mch_id = notifyData.GetValue("sub_mch_id").ToString();

                if (CreateCardOrderFuwushang(sub_mch_id, sub_openid, total_fee, nickName, cardid,
                 transaction_id, EnumPaymentType.小程序微信, companyID))
                {
                    return "<xml><return_code><![CDATA[SUCCESS]]></return_code></xml>";
                }
            }

            return "";
        }
        public bool CreateCardOrderFuwushang(string sub_mch_id, string openid, int total_fee, string nickName, string cardid,
           string transaction_id, EnumPaymentType paymentType, string companyID)
        {
            bool result = false;
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                try
                {
                    string sql = $"UPDATE cfg_card SET overmoney = isnull(overmoney,0) + '{total_fee}' WHERE cardid ={cardid} ";
                    int affectNum = SqlMapper.Execute(conn, sql);
                    if (affectNum == 1)
                    {
                        result = true;
                    }
                }
                catch (Exception e)
                {
                    LogEvent.WebAPILoger.Error("更新充值的金额失败" + e.ToString());
                }
            }
            return result;
        }

        public string CardStartCharge(CardCharge cardCharge)
        {
            LogEvent.WebAPILoger.Info($"微信刷卡启动param={cardCharge.ToString()}");
            bool result = false;
            CardDetail cardInfo = new CardDetail();

            try
            {
                RedisHelper redis5 = new RedisHelper((int)EnumUserRedisNum.HisData);
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    string sql = $"SELECT A.cardid,A.overmoney AS overMoney,A.frozenMoney AS frozenMoney  FROM cfg_card A WHERE A.OneMore = 1 AND A.type = 1 AND A.cardid='{cardCharge.cardid}' ";
                    cardInfo = SqlMapper.Query<CardDetail>(conn, sql).FirstOrDefault();

                }
                int frozenmoney = redis5.HashGet<int>(RedisKey.FrozenMoney, cardCharge.cardid);
                if (cardInfo.overMoney <= frozenmoney + cardInfo.frozenMoney)
                {
                    return "-1";
                }
                string openid = WechatUtility.GetOpenIDByUserToken(cardCharge.userToken);

                //new UserLogic().SaveUserAndPhone(openid, openid, openid);
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                var oldDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, cardCharge.deviceCode);
                string companyID = redis.HashGet<LCO_Station>(RedisKey.Station, WechatUtility.GetStationId(cardCharge.deviceCode)).CompanyID;
                string jkCode = cardCharge.deviceCode.Split('|')[0];
                string orderid = GenerateOrderID();

                //小桔反向对接不支持微信刷卡   此处devicecode=》stationcode  出错 可忽视
                var newDeviceInfo = CreateCurrentDeviceInfo(cardCharge.cardid + "|" + openid, cardCharge.overmoney, cardCharge.nickName, cardCharge.deviceCode,
                    jkCode, "", "", 0, EnumPaymentType.微信刷卡, "", companyID, "", "", orderid);

                if (newDeviceInfo==null)
                {
                    //创建DeviceInfo失败，一般可能是充电策略拿不到。
                    return "3";
                }

                if (oldDeviceInfo == null)
                {
                    result = ChargeAndSaveOrderInfo(newDeviceInfo, redis);
                }
                else
                {
                    if (oldDeviceInfo.Time_end >= long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss")))//同一个设备支付时间一样或比当前还早，可以认为是重复通知
                    {
                        //LogEvent.WebAPILoger.Debug($"WXMINAPayNotifyFuwushang 再次通知 if(wxDeviceInfo.Time_end >= newDeviceInfo.Time_end), wxDeviceInfo.DeviceCode={newDeviceInfo.DeviceCode}, wxDeviceInfo.Time_end={oldDeviceInfo.Time_end}, newDeviceInfo.Time_end={newDeviceInfo.Time_end}, nickName={newDeviceInfo.NickName}，total_fee={newDeviceInfo.Total_fee}，transaction_id={newDeviceInfo.Transaction_id}");
                        result = true;
                    }
                    else if (oldDeviceInfo.Time_end < long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss")))
                    {
                        if (oldDeviceInfo.OrderState != EnumWXOrderState.充电中)
                        {
                            //第一次通知，枪在redis中有缓存，则覆盖上一次充电记录
                            result = ChargeAndSaveOrderInfo(newDeviceInfo, redis);
                        }
                        else
                        {
                            LogEvent.WebAPILoger.Error($"已经充电的桩再次被刷卡 orderstate = {oldDeviceInfo.OrderState},gunstate = {oldDeviceInfo.GunState},deviceCode = {cardCharge.deviceCode}");
                            return "1";
                        }
                    }
                }
                RedisHelper redis6 = new RedisHelper((int)EnumUserRedisNum.HisData);
                redis6.ListRightPush(RedisKey.StartChargeByCard, cardCharge.deviceCode + "|" + cardCharge.cardid + "|40");
                SaveOrderPayment(newDeviceInfo, cardCharge.cardid);
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"微信刷卡启动出错 cardCharge={cardCharge.ToString()}", e);
            }


            if (result == true)
            {
                return "0";
            }
            else
            {
                return "2";
            }
        }

        private void SaveOrderPayment(WXDeviceInfo wxDeviceInfo)
        {
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                orderPaymentType orderPaymentType = new orderPaymentType();
                orderPaymentType.DeviceCode = wxDeviceInfo.DeviceCode;
                orderPaymentType.OrderID = wxDeviceInfo.OrderID;
                orderPaymentType.PaymentType = wxDeviceInfo.PaymentType;
                orderPaymentType.Openid = wxDeviceInfo.Openid.Split('|').LastOrDefault();
                orderPaymentType.Overmoney = wxDeviceInfo.Total_fee;
                orderPaymentType.StartTime = DateTime.Parse(wxDeviceInfo.StartTime);
                redis.HashSet(RedisKey.orderPaymentType, wxDeviceInfo.OrderID, orderPaymentType);
            }
            catch (Exception ex)
            {
                LogEvent.CMUServerLoger.Error($"保存订单支付方式异常！", ex);
            }
        }
        private void SaveOrderPayment(WXDeviceInfo wxDeviceInfo,string cardID)
        {
            try
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                OrderPaymentTypeWithCardID orderPaymentType = new OrderPaymentTypeWithCardID();
                orderPaymentType.DeviceCode = wxDeviceInfo.DeviceCode;
                orderPaymentType.OrderID = wxDeviceInfo.OrderID;
                orderPaymentType.PaymentType = wxDeviceInfo.PaymentType;
                orderPaymentType.CardID = cardID;
                orderPaymentType.Overmoney = wxDeviceInfo.Total_fee;
                orderPaymentType.Openid = wxDeviceInfo.Openid.Split('|').LastOrDefault();
                orderPaymentType.StartTime = DateTime.Parse(wxDeviceInfo.StartTime);
                redis.HashSet(RedisKey.orderPaymentType, wxDeviceInfo.OrderID, orderPaymentType);
            }
            catch (Exception ex)
            {
                LogEvent.CMUServerLoger.Error($"保存订单支付方式异常！", ex);
            }
        }

        public IEnumerable<CardInfo> GetCardInfo(string userToken)
        {
            string openid = WechatUtility.GetOpenIDByUserToken(userToken);

            RedisHelper redis15 = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            RedisHelper redis6 = new RedisHelper((int)EnumUserRedisNum.HisData);
            string phone = redis15.HashGetString(RedisKey.UserAndPhone, openid) == null ? string.Empty : redis15.HashGetString(RedisKey.UserAndPhone, openid);
            if (openid.Length == 11)
            {
                phone = openid;
            }
            StringBuilder sb = new StringBuilder();

            List<string> usableCard = new List<string>();
            var cards = new List<CardInfo>();
            if (phone != null)
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    string getCardsql = $"SELECT A.overmoney/100.00 as overMoney,A.cardid,A.frozenMoney, C.name as companyname,C.logoImageUrl as image FROM cfg_card A,lco_customer B,lco_manufacturer C WHERE A.customerid = b.customerid  AND B.deleteOrUser = 0 AND A.type = 1 AND A.OneMore =1 AND substring(A.cardid,1,9) = C.cardInfo AND B.phone='{phone}' ";
                    cards = SqlMapper.Query<CardInfo>(conn, getCardsql).ToList();
                    string usablePhone = $@"SELECT cardid FROM cfg_card_usablephone WHERE usablePhone LIKE '%{phone}%'";
                    usableCard = SqlMapper.Query<string>(conn, usablePhone).ToList();
                    if (usableCard.Count > 0)
                    {
                        foreach (var item in usableCard)
                        {
                            sb.Append("'");
                            sb.Append(item);
                            sb.Append("',");
                        }
                        sb.Remove(sb.Length - 1, 1);

                        string cardinfosql = $"SELECT A.overmoney/100.00 as overMoney,A.cardid,A.frozenMoney/100.00 AS frozenMoney,C.name AS companyname FROM cfg_card A,lco_customer B,lco_manufacturer C WHERE A.customerid = b.customerid AND    B.deleteOrUser = 0 AND A.type = 1 AND A.OneMore =1 AND substring(A.cardid,1,9) = C.cardInfo AND A.cardid IN ({sb})";

                        cards.AddRange(SqlMapper.Query<CardInfo>(conn, cardinfosql).ToList());
                    }
                }
                foreach (var item in cards)
                {
                    string frozen = redis6.HashGetString(RedisKey.FrozenMoney, item.cardid);
                    if (frozen != null)
                    {
                        item.frozenMoney = Double.Parse(frozen);
                    }
                }
            }
            return cards;
        }

        public CardDetail GetCardDetails(string cardid)
        {
            CardDetail cardInfo = new CardDetail();

            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                DynamicParameters param = new DynamicParameters();
                param.Add("@cardid", cardid);
                string sql = $@"SELECT A.cardid, A.overmoney/100.0 AS overMoney, C.phone AS companyPhone, C.name AS companyname, C.logoImageUrl AS 'image'
                                FROM cfg_card A
                                LEFT JOIN lco_manufacturer C ON LEFT(A.customerid, 9) = C.cardInfo
                                WHERE A.cardid = @cardid AND A.OneMore = 1 AND A.type = 1 ";
                cardInfo = SqlMapper.QueryFirstOrDefault<CardDetail>(conn, sql, param);
                if (cardInfo!=null)
                {
                    string creditPointSQL = $"SELECT isnull(sum(chargecost+servicecost),0) FROM histroy_order_cost WHERE cardid = @cardid ";
                    var creditPoint = SqlMapper.QueryFirst<int>(conn, creditPointSQL, param);
                    cardInfo.integral = creditPoint;

                    RedisHelper redis6 = new RedisHelper((int)EnumUserRedisNum.HisData);
                    string frozen = redis6.HashGetString(RedisKey.FrozenMoney, cardInfo.cardid);
                    if (frozen != null)
                    {
                        cardInfo.frozenMoney = Double.Parse(frozen) / 100.00;
                    }
                }
            }

            return cardInfo;
        }

        public IEnumerable<Order> GetOrderByCardID(string cardid, int MINID)
        {
            if (MINID <= 0)
            {
                MINID = int.MaxValue;
            }
            IEnumerable<Order> orders = new List<Order>();
            using (IDbConnection connection = WechatUtility.CreateDBConnection())
            {
                var sql = $@"SELECT TOP 20 A.id,A.userid,A.orderid,A.stationid,A.stationname,A.deviceCode,A.starttime,A.endtime,
                        A.power,A.chargecost AS ElectricityFee,A.servicecost AS ServiceFee, A.orderduration, A.endreasondesc,A.startsoc,A.endsoc,A.cardid
                        FROM histroy_order_cost A
                        WHERE A.cardid = '{cardid}'and A.id<{MINID} ORDER BY endtime DESC  ";
                orders = SqlMapper.Query<Order>(connection, sql);
            }
            return orders;
        }

        public List<LCO_Station> GetStationByCardID(string cardid)
        {
            List<LCO_Station> stations = new List<LCO_Station>();
            RedisHelper redis14 = new RedisHelper((int)EnumUserRedisNum.WebAPI);
            RedisHelper redis15 = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            List<Authorization_map> stationidList = redis14.HashGet<List<Authorization_map>>(RedisKey.Authorization_map, cardid.Substring(0, 9));
            foreach (var item in stationidList)
            {
                LCO_Station station = redis15.HashGet<LCO_Station>(RedisKey.Station, item.businessid);
                if (station != null)
                {
                    using (IDbConnection conn = WechatUtility.CreateDBConnection())
                    {
                        DateTime lastuseTime = SqlMapper.Query<DateTime>(conn, $"SELECT top 1 endtime FROM histroy_order_cost WHERE stationid = '{station.StationId}' ORDER BY endtime desc").FirstOrDefault();
                        if (lastuseTime != null)
                        {
                            station.LastSuccessTime = WechatUtility.format(lastuseTime);
                        }
                        string ratesql = $"SELECT isnull(avg(rate/1.00),5) FROM lco_faults WHERE stationid = '{station.StationId}' AND Auditing = 0 and iscompany = 0  AND state = 0 AND isexist = 0";
                        station.Score = SqlMapper.Query<double>(conn, ratesql).FirstOrDefault();
                        var files = new ChargeLogic().GetStationImage(station.StationId);
                        if (files != null)
                        {
                            station.Images = new List<string>();
                            for (int i = 0; i < files.Count(); i++)
                            {
                                var a = files[i].Replace(AppDomain.CurrentDomain.BaseDirectory, "").Replace("\\", "/");
                                station.Images.Add(files[i].Replace(AppDomain.CurrentDomain.BaseDirectory, "").Replace("\\", "/"));
                            }
                        }
                    }
                    var stationGunState = redis15.HashGet<StationGunState>(RedisKey.GunStateByStation, station.StationId);

                    station.DCCount = station.DCIdleCount = station.ACSlowCount = station.ACSlowIdleCount = station.ACQuickCount = station.ACQuickIdleCount = 0;
                    if (stationGunState != null && stationGunState.gunState != null)
                    {
                        foreach (var GunStateNum in stationGunState.gunState)
                        {
                            if (GunStateNum.GunType == "直流")
                            {
                                station.DCCount = GunStateNum.GunNum;
                                station.DCIdleCount = GunStateNum.idleGun;
                            }
                            else if (GunStateNum.GunType == "单相交流")
                            {
                                station.ACSlowCount = GunStateNum.GunNum;
                                station.ACSlowIdleCount = GunStateNum.idleGun;
                            }
                            else if (GunStateNum.GunType == "三相交流")
                            {
                                station.ACQuickCount = GunStateNum.GunNum;
                                station.ACQuickIdleCount = GunStateNum.idleGun;
                            }
                        }
                    }
                    station.TodayElectricFeeList = new ChargeLogic().GetTodayElectricityFee(station.StationId, DateTime.Now);
                    station.TodayServiceFeeList = new ChargeLogic().GetTodayServiceFeelist(station.StationId, DateTime.Now);
                    station.CurrentElectricFee = new ChargeLogic().GetCurrentElectricityFee(station.TodayElectricFeeList);
                    station.CurrentServiceFee = new ChargeLogic().GetCurrentServiceFee(station.TodayServiceFeeList);
                    stations.Add(station);
                }
            }
            return stations;
        }

        public bool CheckAndUpdatePhone(CheckPin checkpin)
        {
            bool result = false;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var pinInfo = redis.HashGet<PINInfo>(RedisKey.PIN, checkpin.newPhone);
            string openid = WechatUtility.GetOpenIDByUserToken(checkpin.usertoken);

            if (pinInfo != null && !string.IsNullOrEmpty(pinInfo.PIN)
                && pinInfo.PIN.Equals(checkpin.pin)
                && DateTime.Now.Subtract(pinInfo.LastSendTime.Value).TotalSeconds < 600)
            {
                try
                {
                    redis.HashSet(RedisKey.UserAndPhone, openid, checkpin.newPhone);
                    result = true;
                }
                catch (Exception e)
                {
                    LogEvent.WebAPILoger.Error($"CheckAndUpdatePhone param(oldphone)={checkpin.oldPhone}", e);
                }
            }
            return result;
        }

        private bool DBToRedis(int num)
        {
            bool result = false;
            try
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {
                    string sql = $"UPDATE cfg_timestamp SET updatetime = {DateTime.Now}  WHERE stamptype ={num} ";
                    int effect = SqlMapper.Execute(conn, sql);
                    if (effect == 1)
                    {
                        result = true;
                    }
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error("DBToRedis  更新cfg_timestamp出错", e);
            }

            return result;
        }

        /// <summary>
        /// 下载平台图片到小程序
        /// </summary>
        /// <param name="FileName"></param>
        /// <returns></returns>
        public bool DownLoadSoft(string FileName)
        {

            string DownloadPath = AppDomain.CurrentDomain.BaseDirectory + @"FaultImages\"; //保存的地址
                                                                                           // string FullFilePath = "";//原目标地址
                                                                                           //string FullFilePath = "http://39.104.203.192:8003";//39测试图片地址
            string FullFilePath = "http://mina.xatgood.wang/logoImage/"; //线上图片地址
            bool flag = false;
            try
            {
                if (!Directory.Exists(DownloadPath))
                {
                    Directory.CreateDirectory(DownloadPath);
                }
                if (File.Exists($"DownloadPath + {FileName}"))
                {
                    return true;
                }
                string a = $"{FullFilePath}{FileName}";
                bool aba = File.Exists($"{FullFilePath}{FileName}");

                using (FileStream fs = new FileStream(DownloadPath + "/" + FileName, FileMode.Create))
                {
                    //创建请求
                    WebRequest request = WebRequest.Create(FullFilePath + FileName);
                    //接收响应
                    WebResponse response = request.GetResponse();
                    //输出流
                    Stream responseStream = response.GetResponseStream();
                    byte[] bufferBytes = new byte[10000];//缓冲字节数组
                    int bytesRead = -1;
                    while ((bytesRead = responseStream.Read(bufferBytes, 0, bufferBytes.Length)) > 0)
                    {
                        fs.Write(bufferBytes, 0, bytesRead);
                    }
                    if (fs.Length > 0)
                    {
                        flag = true;
                    }
                    LogEvent.WebAPILoger.Error($"保存{ FileName}");
                    //关闭写入
                    fs.Flush();
                    fs.Close();
                }


            }
            catch (Exception exp)
            {
                LogEvent.WebAPILoger.Error($"DownLoadSoft 从平台下载管理员上传的照片出错", exp);

            }
            return flag;
        }

        public UserBaseInfo GetUserBaseInfo(string usertoken)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            string openid = WechatUtility.GetOpenIDByUserToken(usertoken);
            List<string> openidList = new List<string>();
            string phone = redis.HashGetString(RedisKey.UserAndPhone, openid) == null ? string.Empty : redis.HashGetString(RedisKey.UserAndPhone, openid);
            if (openid.Length == 11)
            {
                phone = openid;
            }
            UserBaseInfo UserBaseInfos = new UserBaseInfo();
            using (IDbConnection conn = WechatUtility.CreateDBConnection())
            {
                if (!string.IsNullOrEmpty(phone))
                {
                    string sql = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                    openidList = conn.Query<string>(sql, conn).ToList();
                }

                string sql1 = $@"SELECT isnull(sum(chargecost+servicecost),0) AS integral FROM histroy_order_cost A WHERE A.userid in (SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}');
                SELECT count(*) AS card FROM lco_customer B,cfg_card C WHERE   B.customerid = C.customerid AND (B.phone='{phone}') AND B.deleteOrUser = 0 and C.OneMore=1 and C.type = 1;
                SELECT count(*) FROM cfg_card_usablephone WHERE usablePhone LIKE '%{phone}%'";
                if (openidList.Count == 0)
                {
                    sql1 = $@"SELECT isnull(sum(chargecost+servicecost),0) AS integral FROM histroy_order_cost A WHERE A.userid ='{openid}';
                SELECT count(*) AS card FROM lco_customer B,cfg_card C WHERE   B.customerid = C.customerid AND (B.phone='{phone}') AND B.deleteOrUser = 0 and C.OneMore=1 and C.type = 1;
                SELECT count(*) FROM cfg_card_usablephone WHERE usablePhone LIKE '%{phone}%'";
                }


                var data = SqlMapper.QueryMultiple(conn, sql1);

                UserBaseInfos.integral = data.Read<int>().FirstOrDefault();
                UserBaseInfos.card = data.Read<int>().FirstOrDefault() + data.Read<int>().FirstOrDefault();

            }
            try
            {
                using (IDbConnection conn = WechatUtility.CreateDBConnection())
                {

                    string sql = $@"SELECT * FROM lco_wxcustomer WHERE openid = '{openid}'";
                    IEnumerable<Wxcustomer> data = SqlMapper.Query<Wxcustomer>(conn, sql);
                    if (data != null && data.Count() > 0)
                    {
                        UserBaseInfos.carType = data.FirstOrDefault().carType;
                        if (data.FirstOrDefault().endurance != null && data.FirstOrDefault().plate != null && data.FirstOrDefault().productionDate != null)
                        {
                            UserBaseInfos.endurance = int.Parse(data.FirstOrDefault().endurance);
                            UserBaseInfos.plate = data.FirstOrDefault().plate;
                            UserBaseInfos.productionDate = data.FirstOrDefault().productionDate;
                        }
                    }
                }
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($@"获取用户信息失败，openid={openid},{e}");
            }
            return UserBaseInfos;
        }
        #endregion

        #region 反向对接

        public bool checkEquipAuth(string connectorID)
        {
            try
            {
                //获取设备运营商

                var manuOperatorID = GetManuOperatorID(connectorID);
                if (manuOperatorID == null)
                    return false;
                if (!GetOperatorData(manuOperatorID, out manuOperator, out teldOperator))
                {
                    LogEvent.GetLoger("EquipAuth").Info($"\n\r 运营商账户不正确！");
                    return false;
                }

                var resultData = GetEquipAuth(connectorID);
                if (resultData == null)
                    return false;

                if (resultData.SuccStat == (int)EnumSuccStat.失败)
                {
                    LogEvent.GetLoger("EquipAuth").Info($"\r\n ConnectorID={connectorID},FailReason={resultData.FailReason}");
                    return false;
                }
            }
            catch (Exception ex)
            {
                LogEvent.GetLoger("EquipAuth").Error(ex.ToString());
                return false;
            }
            return true;

        }

        /// <summary>
        /// 获取到设备运营商
        /// </summary>
        /// <param name="connectorID"></param>
        /// <returns></returns>
        public string GetManuOperatorID(string connectorID)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Reserve);
            string manuOperatorID = null;


            manuOperatorID = redis.HashGetString(RedisKey.ReverseGunMap, connectorID);

            return manuOperatorID;


        }

        /// <summary>
        /// 获取到运营商信息
        /// </summary>
        /// <param name="maunOperatorID"></param>
        /// <returns></returns>
        public bool GetOperatorData(string maunOperatorID, out OperatorData manuOperatorData, out OperatorData teldOperatorData)
        {
            try
            {
                var teldOperatorID = ConfigurationManager.AppSettings["ZDYLOperatorID"];
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Reserve);
                manuOperatorData = null;
                teldOperatorData = null;


                manuOperatorData = redis.HashGet<OperatorData>(RedisKey.ReverseOperator, maunOperatorID);
                teldOperatorData = redis.HashGet<OperatorData>(RedisKey.ReverseOperator, teldOperatorID);

                if (teldOperatorData == null || manuOperatorData == null)
                {
                    return false;
                }
                return true;
            }
            catch (Exception)
            {
                throw;
            }
        }

        /// <summary>
        /// 请求设备认证
        /// </summary>
        /// <param name="connectorID"></param>
        public return_equip_auth GetEquipAuth(string connectorID)
        {
            var dataObject = new JObject();
            dataObject.Add("EquipAuthSeq", teldOperator.OperatorID + GenerateOrderID());
            dataObject.Add("ConnectorID", connectorID.Replace("|", ""));
            var manuOperatorID = GetManuOperatorID(connectorID);

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Reserve);
            if (manuOperatorID != null)
            {
                var result = new return_equip_auth();
                var manuOperator = redis.HashGet<OperatorData>(RedisKey.ReverseOperator, manuOperatorID);
                var Data = PostQuery(JsonConvert.SerializeObject(dataObject), "query_equip_auth", teldOperator, manuOperator, "EquipAuth");
                if (String.IsNullOrEmpty(Data))
                {
                    LogEvent.WebAPILoger.Error($"\n\r 设备认证失败，connectorID={connectorID}");
                    return null;
                }

                result = JsonConvert.DeserializeObject<return_equip_auth>(Data);
                return result;
            }
            return null;
        }

        /// <summary>
        /// 请求参数
        /// </summary>
        /// <param name="Data">序列化后的内容</param>
        /// <param name="functionName">请求的方法名</param>
        /// <returns></returns>
        public string PostQuery(string Data, string functionName, OperatorData teldOperator, OperatorData manuOperator, string logName)
        {
            string result = null;
            try
            {
                //请求次数只有三次机会
                int times = 0;

                #region 对返回结果进行解析
                while (times < 3)
                {
                    var response = GetQueryResponse(teldOperator, manuOperator, logName, functionName, Data);
                    if (response==null)
                    {
                        times++;
                        continue;
                    }

                    switch (response.Ret)
                    {
                        case 0:
                            result = AESUtil.Decrypt(response.Data, manuOperator.DataSecret, manuOperator.DataSecretIV); //成功
                            times = 3;
                            break;

                        case 4001:
                            LogEvent.GetLoger(logName).Error($"{functionName} 失败，OperatorID={manuOperator.OperatorID},  签名错误");
                            times++;
                            break;

                        case 4002:
                            LogEvent.GetLoger(logName).Error($"{functionName} 失败，OperatorID={manuOperator.OperatorID},  Token错误");
                            QueryToServer(teldOperator, manuOperator);
                            times++;
                            break;

                        case 4003:
                            LogEvent.GetLoger(logName).Error($"{functionName} 失败，OperatorID={manuOperator.OperatorID},  Post参数不合法，缺少必要参数");
                            times++;
                            break;

                        case 4004:
                            LogEvent.GetLoger(logName).Error($"{functionName} 失败，OperatorID={manuOperator.OperatorID}, 业务参数不合法");
                            times++;
                            break;

                        default:
                            LogEvent.GetLoger(logName).Error($"{functionName} 失败，OperatorID={manuOperator.OperatorID},返回未定义。response.Ret={response.Ret}.");
                            times++;
                            break;
                    }
                }
                #endregion
            }
            catch (Exception ex)
            {
                string ex_teldOperator = teldOperator == null ? "null" : teldOperator.ToString();
                string ex_manuOperator = manuOperator == null ? "null" : manuOperator.ToString();
                LogEvent.WebAPILoger.Error($"小桔设备请求失败！！！！！！！！！！！！！！Data={Data},functionName={functionName},teldOperator={ex_teldOperator},manuOperator={ex_manuOperator},logName={logName}。",ex);
            }
            return result;
        }

        /// <summary>
        /// 返回值
        /// </summary>
        /// <param name="teldOperator"></param>
        /// <param name="manuOperator"></param>
        /// <param name="logName"></param>
        /// <param name="functionName"></param>
        /// <param name="Data"></param>
        /// <returns></returns>
        private ReturnData GetQueryResponse(OperatorData teldOperator, OperatorData manuOperator, string logName, string functionName, string Data)
        {
            var token = GetToken(teldOperator, manuOperator);
            if (token == null)
            {
                LogEvent.WebAPILoger.Error($"{functionName}  请求token失败");
                return null;
            }

            //拿到数据
            var Header = ConstructBody(Data, teldOperator, manuOperator);

            var url = manuOperator.NotifyURL.Replace("notification_stationStatus", functionName);

            //拿到返回值
            ReturnData response = PostBody(Header, url, token, logName);
            return response;

        }

        /// <summary>
        /// Token信息是以manuOperatorID为key存储存储的。我们的token
        /// </summary>
        /// <param name="teldOperator"></param>
        /// <param name="manuOperator"></param>
        /// <returns></returns>
        public string GetToken(OperatorData teldOperator, OperatorData manuOperator)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Reserve);
            LoginToken logintoken = null;


            logintoken = redis.HashGet<LoginToken>(RedisKey.ReverseQueryToken, teldOperator.OperatorID);


            //拿到了Token且没有过去
            if (logintoken != null && logintoken.ExpireTime > DateTime.Now)
            {
                return logintoken.Token;
            }
            logintoken = QueryToken(teldOperator, manuOperator);
            //只请求两次
            if (logintoken == null)
                logintoken = QueryToken(teldOperator, manuOperator);
            if (logintoken == null)
                return null;

            redis.HashSet(RedisKey.ReverseQueryToken, teldOperator.OperatorID, logintoken);

            return logintoken.Token;

        }

        /// <summary>
        /// 发送请求
        /// </summary>
        /// <param name="teldOperator"></param>
        /// <param name="manuOperator"></param>
        /// <returns></returns>
        public LoginToken QueryToken(OperatorData teldOperator, OperatorData manuOperator)
        {
            try
            {
                var queryObject = new JObject();
                queryObject.Add("OperatorID", teldOperator.OperatorID);
                queryObject.Add("OperatorSecret", teldOperator.OperatorSecret);
                string Data = JsonConvert.SerializeObject(queryObject);

                var Header = ConstructBody(Data, teldOperator, manuOperator);

                var url = manuOperator.NotifyURL.Replace("notification_stationStatus", "query_token");

                //请求Token
                ReturnData response = PostBody(Header, url, null, "Query_Token");

                if (response.Ret != 0)
                {
                    LogEvent.GetLoger("Query_Token").Error($"Ret={response.Ret},请求的Data={Data}");
                    return null;
                }
                string returnData = AESUtil.Decrypt(response.Data, manuOperator.DataSecret, manuOperator.DataSecretIV);

                QueryToken returnToken = JsonConvert.DeserializeObject<QueryToken>(returnData);
                var loginToken = new LoginToken(returnToken, manuOperator.OperatorSecret);

                return loginToken;

            }
            catch (Exception ex)
            {
                LogEvent.GetLoger("Query_Token").Error(ex.ToString());
            }
            return null;
        }


        /// <summary>
        /// 组织body并签名
        /// </summary>
        /// <param name="Data"></param>
        /// <param name="teldOperator"></param>
        /// <param name="manuOperator"></param>
        /// <returns></returns>
        public HeaderData ConstructBody(string Data, OperatorData teldOperator, OperatorData manuOperator)
        {
            var result = new HeaderData();
            result.OperatorID = teldOperator.OperatorID;
            result.TimeStamp = DateTime.Now.ToString("yyyyMMddHHmmss");
            result.Seq = "0001";
            //Data数据加密
            result.Data = AESUtil.Encrypt(Data, manuOperator.DataSecret, manuOperator.DataSecretIV);
            var str = new StringBuilder();
            str.Append(result.OperatorID);
            str.Append(result.Data);
            str.Append(result.TimeStamp);
            str.Append(result.Seq);
            //签名
            result.Sig = HMacMD5.getHmacMd5Str(manuOperator.SigSecret, str.ToString());
            return result;

        }

        /// <summary>
        /// 发送命令
        /// </summary>
        /// <param name="header"></param>
        /// <param name="url"></param>
        /// <param name="token"></param>
        /// <returns></returns>
        public ReturnData PostBody(HeaderData Header, string url, string token, string logName)
        {
            try
            {
                Dictionary<string, string> headers = null;
                if (token != null)
                {
                    headers = new Dictionary<string, string>();
                    headers.Add("Authorization", "Bearer " + token);
                }


                var data = JsonConvert.SerializeObject(Header);
                for (var i = 0; i < 3; i++)
                {
                    var response =
                        HttpService.Post(data, url, headers);
                    if (response != null)
                    {
                        var result = JsonConvert.DeserializeObject<ReturnData>(response);
                        //系统测试的数据
                        if (result.Ret == -1 || String.IsNullOrEmpty(result.Data))
                        {
                            LogEvent.GetLoger(logName).Error($"{url},Ret={result.Ret},Data={result.Data}");
                            continue;
                        }
                        return result;
                    }
                }
            }
            catch (Exception ex)
            {
                string ex_header = Header == null ? "null" : Header.ToString();
                LogEvent.GetLoger(logName).Error($"Header={ex_header},url={url},token={token},logName={logName}。", ex);
            }
            return null;


        }

        /// <summary>
        /// 获取充电策略
        /// </summary>
        /// <param name="ConnectorID"></param>
        /// <returns></returns>
        public List<policyInfo> GetBusinessPolicy(string ConnectorID)
        {
            var result = new List<policyInfo>();
            try
            {

                var manuOperatorID = GetManuOperatorID(ConnectorID);
                if (String.IsNullOrEmpty(manuOperatorID))
                    return null;
                if (!GetOperatorData(manuOperatorID, out manuOperator, out teldOperator))
                {
                    LogEvent.GetLoger("BusinessPolicy").Info($"\r\n ConnectorID={ConnectorID}");
                    return null;
                }

                var resultData = GetXiaoJu_query_equip_business_policy(ConnectorID);

                if (resultData == null)
                    return null;
                if (resultData.SuccStat == (int)EnumSuccStat.失败)
                {
                    LogEvent.GetLoger("BusinessPolicy").Info($"\r\n  ConnectorID={ConnectorID} 获取策略失败,FailReason={resultData.FailReason.ToString()}");
                    return null;
                }

                result = resultData.PolicyInfos;
                return result;

            }
            catch (Exception ex)
            {
                LogEvent.GetLoger("BusinessPolicy").Info($"\r\n ConnectorID={ConnectorID}.", ex);
                return null;

            }
        }

        /// <summary>
        /// 发送请求
        /// </summary>
        private return_business_policy GetXiaoJu_query_equip_business_policy(string ConnectorID)
        {

            var DataObject = new JObject();
            DataObject.Add("EquipBizSeq", teldOperator.OperatorID + GenerateOrderID());
            DataObject.Add("ConnectorID", ConnectorID.Replace("|", ""));

            var Data = PostQuery(JsonConvert.SerializeObject(DataObject), "query_equip_business_policy", teldOperator, manuOperator, "BusinessPolicy");
            if (String.IsNullOrEmpty(Data))
            {
                LogEvent.GetLoger("BusinessPolicy").Info($"\n\r 获取设备策略失败，connectorID={ConnectorID}");
                return null;
            }

            var result = JsonConvert.DeserializeObject<return_business_policy>(Data);
            return result;

        }

        /// <summary>
        /// 转化为我们的格式
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        private List<ChargeStrategy> ConstructChargeStrategy(List<policyInfo> policyList)
        {
            var ChargeStrategyList = new List<StrategyInfo>();
            policyList = policyList.OrderBy(p => p.StartTime).ToList();
            foreach (var item in policyList)
            {
                var chargeStrategy = new StrategyInfo();
                chargeStrategy.StartTime = ChangeToSTime(item.StartTime);
                chargeStrategy.Price = (int)((item.ElecPrice + item.ServicePrice) * 1000);
                chargeStrategy.Valid = true;
                ChargeStrategyList.Add(chargeStrategy);
            }

            for (var i = 0; i < ChargeStrategyList.Count; i++)
            {
                var item = ChargeStrategyList[i];
                if (i < ChargeStrategyList.Count - 1)
                {
                    item.EndTime = ChargeStrategyList[i + 1].StartTime;
                }
                else
                {
                    item.EndTime = ChangeToSTime("235959");
                }
            }
            var ChargeStrategy = new ChargeStrategy();
            ChargeStrategy.UseTime = DateTime.Now.AddDays(-1);
            ChargeStrategy.EndUseTime = DateTime.Now.AddDays(2);
            ChargeStrategy.Strategy = ChargeStrategyList;

            var result = new List<ChargeStrategy>();
            result.Add(ChargeStrategy);
            return result;


        }

        /// <summary>
        /// 获取到开始时间的STime
        /// </summary>
        /// <param name="startTime"></param>
        /// <returns></returns>
        STimeInfo ChangeToSTime(string startTime)
        {
            var result = new Models.STimeInfo();
            result.Hour = int.Parse(startTime.Substring(0, 2));
            result.Minute = int.Parse(startTime.Substring(2, 2));
            result.Second = int.Parse(startTime.Substring(4, 2));
            return result;

        }


        /// <summary>
        /// 小桔设备下发充电命令
        /// </summary>
        /// <param name="ConnectorID"></param>
        /// <param name="orderID"></param>
        public bool XiaojuDeviceStartCharge(string ConnectorID, string orderID)
        {
            string loggerName = "ChargeStart";
            try
            {
                var manuOperatorID = GetManuOperatorID(ConnectorID);
                if (manuOperatorID == null)
                    return false;
                if (!GetOperatorData(manuOperatorID, out manuOperator, out teldOperator))
                {
                    LogEvent.GetLoger(loggerName).Info($"\r\n ConnectorID={ConnectorID}");
                    return false;
                }

                var chargeResult = GetXiaoju_query_start_charge(ConnectorID, orderID, loggerName);

                if (chargeResult == null)
                {
                    LogEvent.GetLoger(loggerName).Error($"小桔启动返回为空。ConnectorID={ConnectorID}，orderID={orderID}，manuOperatorID={manuOperatorID}。");
                    return false;
                }
                if (chargeResult.SuccStat != (int)EnumSuccStat.成功)
                {
                    LogEvent.GetLoger(loggerName).Error($"\r\n 启动失败 ConnectorID={ConnectorID},orderID={orderID} ,FailReason={chargeResult.FailReason}");
                }
                //SaveActiveOrder(ConnectorID, orderID);
                return true;

            }
            catch (Exception ex)
            {
                LogEvent.GetLoger(loggerName).Info($"\r\n ConnectorID={ConnectorID},orderID={orderID}。", ex);
                throw;
            }
        }

        /// <summary>
        /// 发送请求
        /// </summary>
        /// <param name="connectorID"></param>
        /// <param name="orderID"></param>
        /// <returns></returns>
        private return_start_charge GetXiaoju_query_start_charge(string connectorID, string orderID,string loggerName)
        {

            var DataObject = new JObject();
            DataObject.Add("StartChargeSeq", orderID);
            DataObject.Add("ConnectorID", connectorID.Replace("|", ""));
            DataObject.Add("QRCode", "");

            var Data = PostQuery(JsonConvert.SerializeObject(DataObject), "query_start_charge", teldOperator, manuOperator, loggerName);
            if (String.IsNullOrEmpty(Data))
            {
                LogEvent.GetLoger(loggerName).Error($"GetXiaoju_query_start_charge请求出错。ConnectorID={connectorID}，orderID={orderID}。");
                return null;
            }
            var result = JsonConvert.DeserializeObject<return_start_charge>(Data);
            return result;
        }

        /// <summary>
        /// 停止命令
        /// </summary>
        /// <param name="ConnectorID"></param>
        /// <param name="orderID"></param>
        /// <returns></returns>
        public bool ChargeStop(string ConnectorID, string orderID)
        {
            try
            {

                var manuOperatorID = GetManuOperatorID(ConnectorID);
                if (manuOperatorID == null)
                    return false;
                if (!GetOperatorData(manuOperatorID, out manuOperator, out teldOperator))
                {
                    LogEvent.GetLoger("ChargeStop").Info($"\r\n ConnectorID={ConnectorID}");
                    return false;
                }

                var chargeResult = SendPostStop(ConnectorID, orderID);

                if (chargeResult == null)
                    return false;
                if (chargeResult.SuccStat != (int)EnumSuccStat.成功)
                {
                    LogEvent.GetLoger("ChargeStop").Info($"\r\n 停止失败 ConnectorID={ConnectorID},orderID={orderID} ,FailReason={chargeResult.FailReason}");
                }

                return true;

            }
            catch (Exception ex)
            {
                LogEvent.GetLoger("ChargeStop").Info($"\r\n ConnectorID={ConnectorID},orderID={orderID} \r\n{ex.ToString()}");
                throw;
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="ConnectorID"></param>
        /// <param name="orderID"></param>
        /// <returns></returns>
        private return_stop_charge SendPostStop(string ConnectorID, string orderID)
        {
            var DataObject = new JObject();
            DataObject.Add("StartChargeSeq", orderID);
            DataObject.Add("ConnectorID", ConnectorID.Replace("|", ""));
            var manuOperatorID = GetManuOperatorID(ConnectorID);
            var Data = PostQuery(JsonConvert.SerializeObject(DataObject), "query_stop_charge", teldOperator, manuOperator, "BusinessPolicy");
            if (String.IsNullOrEmpty(Data))
            {
                LogEvent.GetLoger("ChargeStop").Info($"\n\r 获取设备策略失败，connectorID={ConnectorID}");
                return null;
            }

            var result = JsonConvert.DeserializeObject<return_stop_charge>(Data);
            return result;
        }


        /// <summary>
        /// 查询到状态
        /// </summary>
        /// <param name="ConnectorID"></param>
        /// <param name="OrderID"></param>
        /// <returns></returns>
        private return_charge_status QueryOrderStatus(string ConnectorID, string OrderID)
        {
            var DataObject = new JObject();
            DataObject.Add("StartChargeSeq", OrderID);


            var Data = PostQuery(JsonConvert.SerializeObject(DataObject), "query_equip_charge_status", teldOperator, manuOperator, "OrderStatus");

            if (String.IsNullOrEmpty(Data))
            {
                LogEvent.GetLoger("OrderStatus").Info($"\n\r 获取订单状态失败，connectorID={ConnectorID}");
                return null;
            }
            LogEvent.GetLoger("OrderStatus").Info($"\n\r 获取到的订单状态为 ： \n\r {Data}");
            var result = JsonConvert.DeserializeObject<return_charge_status>(Data);
            return result;
        }

        public LoginToken QueryToServer(OperatorData teldOperator, OperatorData manuOperator)
        {
            var logintoken = QueryToken(teldOperator, manuOperator);
            //只请求两次
            if (logintoken == null)
                logintoken = QueryToken(teldOperator, manuOperator);
            if (logintoken == null)
                return null;
            SaveToRedis(logintoken, manuOperator);
            return logintoken;
        }

        /// <summary>
        /// 保存到Redis
        /// </summary>
        /// <returns></returns>
        public void SaveToRedis(LoginToken logintoken, OperatorData manuOperator)
        {
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.Reserve);
            redis.HashSet(RedisKey.ReverseQueryToken, manuOperator.OperatorID, logintoken);
        }
        #endregion

        #region i电启动方法

        public return_start_charge IDianStartDevice(string deviceCode,string orderID)
        {
            return_start_charge chargeResult = new return_start_charge();

            if (string.IsNullOrEmpty(deviceCode))
            {
                //设备ID为空
                chargeResult.ConnectorID = deviceCode;
                chargeResult.StartChargeSeq = orderID;
                chargeResult.StartChargeSeqStat = 0;
                chargeResult.SuccStat = (int)EnumSuccStat.失败;
                chargeResult.FailReason = EnumStartFailReason.非小桔设备;

                return chargeResult;
            }

            deviceCode = WechatUtility.XiaoJuFilter(deviceCode);
            RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve);
            bool isXiaojuDevice = reverse.HashExists(RedisKey.ReverseGunMap, deviceCode);
            if (isXiaojuDevice != true)
            {
                //非小桔设备
                chargeResult.ConnectorID = deviceCode;
                chargeResult.StartChargeSeq = orderID;
                chargeResult.StartChargeSeqStat = 0;
                chargeResult.SuccStat = (int)EnumSuccStat.失败;
                chargeResult.FailReason = EnumStartFailReason.非小桔设备;

                return chargeResult;
            }

            chargeResult = CreateRedisOrderFuwushangForIDian(deviceCode, orderID);

            return chargeResult;
        }

        public return_start_charge CreateRedisOrderFuwushangForIDian(string deviceCode, string orderID)
        {
            return_start_charge chargeResult = new return_start_charge();

            if (string.IsNullOrEmpty(orderID) || orderID.Length != 18 || orderID.Last() != 'i')
            {
                LogEvent.GetLoger("iDian").Error($"CreateRedisOrderFuwushangForIDian i电给的订单号({orderID})不符合规则.");
                orderID = teldOperator.OperatorID + GenerateOrderID().Remove(17, 1) + "i";
            }

            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
            var oldDeviceInfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);

            LCO_Station station = WechatUtility.GetStation(deviceCode);
            if (station==null)
            {
                chargeResult.ConnectorID = deviceCode;
                chargeResult.StartChargeSeq = orderID;
                chargeResult.StartChargeSeqStat = 0;
                chargeResult.SuccStat = (int)EnumSuccStat.失败;
                chargeResult.FailReason = EnumStartFailReason.找不到对应的运营商ID;
                return chargeResult;
            }

            WXDeviceInfo newDeviceInfo = CreateCurrentDeviceInfo("iDian", 99999, "iDian", deviceCode, deviceCode.Split('|').FirstOrDefault(),
                "transaction_id", "out_trade_no", long.Parse(DateTime.Now.ToString("yyyyMMddHHmmss")), 
                EnumPaymentType.i电, "prepayID", station.CompanyID, "tid", "iDian", orderID);

            if (newDeviceInfo == null)
            {
                //创建DeviceInfo失败，一般可能是充电策略拿不到。
                LogEvent.GetLoger("iDian").Error($"CreateRedisOrderFuwushangForIDian:创建DeviceInfo失败，一般可能是充电策略拿不到。deviceCode={deviceCode},orderID={orderID}.");
                chargeResult.ConnectorID = deviceCode;
                chargeResult.StartChargeSeq = orderID;
                chargeResult.StartChargeSeqStat = 0;
                chargeResult.SuccStat = (int)EnumSuccStat.失败;
                chargeResult.FailReason = EnumStartFailReason.其他;
                return chargeResult;
            }

            chargeResult = ChargeAndSaveOrderInfoForIDian(newDeviceInfo);

            return chargeResult;
        }

        private return_start_charge ChargeAndSaveOrderInfoForIDian(WXDeviceInfo newDeviceInfo)
        {
            return_start_charge chargeResult = new return_start_charge();

            RedisHelper reverse = new RedisHelper((int)EnumUserRedisNum.Reserve);
            bool isXiaojuDevice = reverse.HashExists(RedisKey.ReverseGunMap, newDeviceInfo.DeviceCode);
            if (isXiaojuDevice == true)
            {
                //小桔设备启动流程
                chargeResult = XiaojuDeviceStartChargeForIDian(newDeviceInfo.DeviceCode, newDeviceInfo.OrderID);
            }
            else
            {
                //非小桔设备正常流程
                chargeResult.ConnectorID = newDeviceInfo.DeviceCode;
                chargeResult.StartChargeSeq = newDeviceInfo.OrderID;
                chargeResult.StartChargeSeqStat = 0;
                chargeResult.SuccStat = (int)EnumSuccStat.失败;
                chargeResult.FailReason = EnumStartFailReason.非小桔设备;
            }

            if (chargeResult.SuccStat == (int)EnumSuccStat.成功)
            {
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin, RedisConnectionString);
                redis.HashSet(RedisKey.DeviceInfo, newDeviceInfo.DeviceCode, newDeviceInfo);
                //SaveOrderInfo(newDeviceInfo, redis);//将设备和用户ID管理，i电启动不需要
                SaveOrderPayment(newDeviceInfo);
                SendPaymentType(newDeviceInfo);
                SaveChargeingDevice(newDeviceInfo.DeviceCode);
                SaveActiveOrderRedis(newDeviceInfo);
            }
            return chargeResult;
        }

        public return_start_charge XiaojuDeviceStartChargeForIDian(string ConnectorID, string orderID)
        {
            return_start_charge chargeResult = new return_start_charge();
            chargeResult.ConnectorID = ConnectorID;
            chargeResult.StartChargeSeq = orderID;
            chargeResult.StartChargeSeqStat = 0;
            chargeResult.SuccStat = (int)EnumSuccStat.失败;

            string loggerName = "iDian";
            try
            {
                var manuOperatorID = GetManuOperatorID(ConnectorID);
                if (string.IsNullOrEmpty(manuOperatorID))
                {
                    LogEvent.GetLoger(loggerName).Error($"启动失败 设备={ConnectorID},orderID={orderID},找不到对应的运营商ID");
                    chargeResult.FailReason = EnumStartFailReason.找不到对应的运营商ID;
                    return chargeResult;
                }
                    
                if (!GetOperatorData(manuOperatorID, out manuOperator, out teldOperator))
                {
                    LogEvent.GetLoger(loggerName).Error($"启动失败 设备={ConnectorID},orderID={orderID},对应的运营商ID({manuOperatorID})找不到对应的运营商信息");
                    chargeResult.FailReason = EnumStartFailReason.找不到对应的运营商信息;
                    return chargeResult;
                }

                chargeResult = GetXiaoju_query_start_chargeForIDian(ConnectorID, orderID, loggerName);
                if (chargeResult == null || chargeResult.SuccStat != (int)EnumSuccStat.成功)
                {
                    string failReason = chargeResult == null ? "小桔充电出错" : chargeResult.FailReason.ToString();
                    LogEvent.GetLoger(loggerName).Error($"启动失败 设备={ConnectorID},orderID={orderID},FailReason={failReason}");
                }
            }
            catch (Exception ex)
            {
                LogEvent.GetLoger(loggerName).Info($"启动失败 设备={ConnectorID},orderID={orderID}。", ex);
                chargeResult.FailReason = EnumStartFailReason.其他;
            }

            return chargeResult;
        }

        private return_start_charge GetXiaoju_query_start_chargeForIDian(string connectorID, string orderID, string loggerName)
        {

            var DataObject = new JObject();
            DataObject.Add("StartChargeSeq", orderID);
            DataObject.Add("ConnectorID", connectorID.Replace("|", ""));
            DataObject.Add("QRCode", "");

            var Data = PostQuery(JsonConvert.SerializeObject(DataObject), "query_start_charge", teldOperator, manuOperator, loggerName);
            if (String.IsNullOrEmpty(Data))
            {
                LogEvent.GetLoger(loggerName).Error($"GetXiaoju_query_start_chargeForIDian请求出错。ConnectorID={connectorID}，orderID={orderID}。");
                return null;
            }
            var result = JsonConvert.DeserializeObject<return_start_charge>(Data);
            return result;
        }

        #endregion

        #region 从webapi获取策略
        private Strategy GetEffectiveFee(string stationID, DateTime startTime, EnumPaymentType paymentType, string discountID)
        {
            Strategy strategy = null;
            try
            {
                if (!StrategyAPI.EndsWith("/"))
                {
                    StrategyAPI = StrategyAPI + "/";
                }
                var uri = StrategyAPI + $"GetEffectiveFee?stationId={stationID}&userId={discountID}&platId={(int)paymentType}&time={startTime:yyyy-MM-dd HH:mm:ss}";
                LogEvent.WebAPILoger.Info($"GetEffectiveFee uri={uri}");

                HttpResponseMessage apiResult = new HttpResponseMessage();
                using (var httpClient = new HttpClient())
                {
                    apiResult = httpClient.GetAsync(uri).Result;
                }

                if (apiResult.IsSuccessStatusCode)
                {
                    string apiResultString = apiResult.Content.ReadAsStringAsync().Result;
                    JObject jObj = JObject.Parse(apiResultString);
                    if (jObj.GetValue("code") != null && jObj.GetValue("code").ToString().Equals("200"))
                    {
                        strategy = JsonConvert.DeserializeObject<Strategy>(jObj.GetValue("data").ToString(Formatting.None));
                    }
                    else
                    {
                        LogEvent.WebAPILoger.Error($"GetEffectiveFee Error = {apiResultString}。");
                    }
                }
                else
                {
                    LogEvent.WebAPILoger.Error($"GetEffectiveFee Error apiResult = {apiResult}。");
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetEffectiveFee Error。", ex);
            }
            
            return strategy;
        }
        #endregion
    }
}

